#!/usr/bin/env python3

"""
omlabs: model-simulated sea ice vs. NSIDC obs
"""

__all__ = ["parse", "read", "calculate", "plot", "run", "parse_and_run"]

import argparse
import copy
import time
import warnings

import cartopy.crs as ccrs
import cartopy.feature
import intake
import matplotlib as mpl
import matplotlib.pyplot as plt
import pkg_resources as pkgr
from matplotlib.lines import Line2D

from omlabs.om4common import (
    annual_cycle,
    curv_to_curv,
    date_range,
    image_handler,
    open_intake_catalog,
    standard_grid_cell_area,
)
from omlabs.om4parser import default_diag_parser

warnings.filterwarnings("ignore", message=".*csr_matrix.*")
warnings.filterwarnings("ignore", message=".*dates out of range.*")

import io
import os
import shutil
import tempfile

import numpy as np
import palettable
import xarray as xr


def parse(cliargs=None, template=False):
    description = """Plot sea ice vs. NSIDC"""

    parser = default_diag_parser(
        description=description, template=template, exclude=["basin", "topog"]
    )

    parser.add_argument(
        "--month",
        type=str,
        default="March",
        help="Month to analyze. Deafult is March",
    )

    parser.add_argument(
        "--region",
        type=str,
        default="nh",
        help="Region for analysis. Default is nh. Options are nh and sh",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


# def read(infile, obsfile, static):
def read(dictArgs):
    """Function to read in the data. Returns xarray datasets"""

    infile = dictArgs["infile"]

    # Open ice model output and the static file
    ds = xr.open_mfdataset(infile, combine="by_coords")
    if "siconc" in ds.variables:
        ds["CN"] = ds["siconc"]
        if ds["CN"].max() > 1.0:
            ds["CN"] = ds["CN"] / 100.0
    else:
        ds["CN"] = ds["CN"].sum(dim="ct")

    # Detect if we are native grid or 1x1
    if (ds.CN.shape[-2] == 180) and (ds.CN.shape[-1] == 360):
        standard_grid = True
    else:
        standard_grid = False

    if dictArgs["config"] is not None:
        # use dataset from catalog, either from command line or default
        cat = open_intake_catalog(dictArgs["platform"], dictArgs["config"])
        if standard_grid is True:
            dstatic = cat["ice_static_1x1"].to_dask()
        else:
            dstatic = cat["ice_static"].to_dask()

    # Override static file if provided
    if dictArgs["static"] is not None:
        dstatic = xr.open_dataset(dictArgs["static"])

    # Append static fields to the return Dataset
    if standard_grid is True:
        _lon = np.array(dstatic["lon"].to_masked_array())
        _lat = np.array(dstatic["lat"].to_masked_array())
        X, Y = np.meshgrid(_lon, _lat)
        ds["GEOLON"] = xr.DataArray(X, dims=["lat", "lon"])
        ds["GEOLAT"] = xr.DataArray(Y, dims=["lat", "lon"])
        _AREA = standard_grid_cell_area(_lat, _lon)
        _MASK = np.array(dstatic["mask"].fillna(0.0).to_masked_array())
        _AREA = _AREA * _MASK
        ds["CELL_AREA"] = xr.DataArray(_AREA, dims=["lat", "lon"])
        ds["AREA"] = xr.DataArray(_AREA, dims=["lat", "lon"])
        ds = ds.rename({"lon": "x", "lat": "y"})
    else:
        ds["CELL_AREA"] = dstatic["CELL_AREA"]
        ds["GEOLON"] = dstatic["GEOLON"]
        ds["GEOLAT"] = dstatic["GEOLAT"]
        ds["AREA"] = dstatic["CELL_AREA"] * 4.0 * np.pi * (6.378e6 ** 2)

    # Get Valid Mask
    valid_mask = np.where(ds["CELL_AREA"] == 0.0, True, False)

    # Open observed SIC on 25-km EASE grid (coords already named lat and lon)
    if dictArgs["obsfile"] is not None:
        dobs = xr.open_dataset(dictArgs["obsfile"])
    else:
        cat = open_intake_catalog(dictArgs["platform"], "obs")
        dobs = cat[f"NSIDC_{dictArgs['region'].upper()}_monthly"].to_dask()

    # Close the static file (no longer used)
    dstatic.close()

    return ds, dobs, valid_mask


def calculate(ds, dobs, region="nh"):
    """Function to calculate sea ice parameters"""

    # Container dictionaries to hold results
    model = xr.Dataset()
    obs = xr.Dataset()

    # Add coordinates
    model["GEOLON"] = ds["GEOLON"]
    model["GEOLAT"] = ds["GEOLAT"]
    model = model.rename({"GEOLON": "lon", "GEOLAT": "lat"})
    obs["lon"] = dobs["lon"]
    obs["lat"] = dobs["lat"]

    # Create annual cycle climatology
    model["ac"] = annual_cycle(ds, "CN")
    obs["ac"] = annual_cycle(dobs, "sic")

    # Regrid the observations to the model grid (for plotting)
    regridded = curv_to_curv(obs, model, reuse_weights=False)

    # Calculate area and extent
    if region == "nh":
        model["area"] = xr.where(ds["GEOLAT"] > 0.0, model["ac"] * ds.AREA, 0.0)
        model["ext"] = xr.where(
            (model["ac"] > 0.15) & (ds["GEOLAT"] > 0.0), ds.AREA, 0.0
        )
    elif region == "sh":
        model["area"] = xr.where(ds["GEOLAT"] < 0.0, model["ac"] * ds.AREA, 0.0)
        model["ext"] = xr.where(
            (model["ac"] > 0.15) & (ds["GEOLAT"] < 0.0), ds.AREA, 0.0
        )
    else:
        raise ValueError(f"Unknown region {region}. Option are nh or sh")

    # Ensure dims are in the correct order
    model["area"] = model["area"].transpose("month", ...)
    model["ext"] = model["ext"].transpose("month", ...)

    # Sum to get model area and extent
    model["area"] = model["area"].sum(axis=(-2, -1)) * 1.0e-12
    model["ext"] = model["ext"].sum(axis=(-2, -1)) * 1.0e-12

    # Get obs model and extent
    obs["area"] = obs["ac"] * dobs.areacello
    obs["area"] = obs["area"].transpose("month", ...)
    obs["area"] = obs["area"].sum(axis=(-2, -1)) * 1.0e-12

    obs["ext"] = xr.where(obs["ac"] > 0.15, dobs.areacello, 0.0)
    obs["ext"] = obs["ext"].transpose("month", ...)
    obs["ext"] = obs["ext"].sum(axis=(-2, -1)) * 1.0e-12

    # Get tuple of start year and end years for model and observations
    model.attrs["time"] = date_range(ds)
    obs.attrs["time"] = date_range(dobs)

    return model, obs, regridded


def _plot_annual_cycle(ax, _mod, _obs, roll=0):
    """Creates an anual cycle subplot panel"""
    _months = np.array(["J", "F", "M", "A", "M", "J", "J", "A", "S", "O", "N", "D"])

    # Shift annual cycle
    _months = np.roll(_months, roll)
    _obs = np.roll(_obs, roll)
    _mod = np.roll(_mod, roll)

    # Add wrap-around point
    _months = np.append(_months, _months[0])
    _obs = np.append(_obs, _obs[0])
    _mod = np.append(_mod, _mod[0])

    # Plot the data
    t = np.arange(0, len(_mod))
    ax.plot(t, _mod, "r", label="model")
    ax.plot(t, _obs, "k", label="obs")

    # Format the plot
    ax.set_xticks(np.arange(0, len(_obs)))
    ax.set_xticklabels(_months)
    plt.legend()


def _plot_map_panel(
    ax,
    x,
    y,
    plotdata,
    cmap="Blues_r",
    vmin=0.01,
    vmax=100.0,
    extent=[-180, 180, 50, 90],
    contour=True,
):
    """Function to plot a map-based sea ice panel"""
    cmap = copy.copy(mpl.cm.get_cmap(cmap))
    cmap.set_bad(color="#555555", alpha=1)
    ax.set_extent(extent, ccrs.PlateCarree())
    cb = ax.pcolormesh(
        x, y, plotdata, transform=ccrs.PlateCarree(), cmap=cmap, vmin=vmin, vmax=vmax
    )
    if contour is True:
        cs2 = ax.contour(
            x, y, plotdata, transform=ccrs.PlateCarree(), levels=[15.0], colors=["r"]
        )
    return cb


def plot(model, obs, regridded, valid_mask, label=None, region="nh", month="March"):
    """Function to make sea ice plot"""

    # Get integer index of the requested month
    month_index = int(time.strptime(month, "%B").tm_mon) - 1

    # Setup figure canvas
    fig = plt.figure(figsize=(11, 8.5))

    # Define projection
    if region == "nh":
        proj = ccrs.NorthPolarStereo(central_longitude=-100)
        extent = [-180, 180, 50, 90]
    elif region == "sh":
        proj = ccrs.SouthPolarStereo(central_longitude=-100)
        extent = [-180, 180, -50, -90]
    else:
        raise ValueError(f"Unknown region {region}. Option are nh or sh")

    # All maps are plotted on the model grid
    x = np.array(model["lon"].to_masked_array())
    y = np.array(model["lat"].to_masked_array())

    # Top left panel - model map of sea ice
    ax = plt.subplot(2, 3, 1, projection=proj)
    plotdata = (model["ac"][month_index] * 100.0).to_masked_array()
    plotdata = np.ma.masked_where(valid_mask, plotdata)
    cb1 = _plot_map_panel(ax, x, y, plotdata, extent=extent)
    ax.set_title(f"Model - Years {model.time[0]} to {model.time[1]}")
    fig.colorbar(
        cb1, orientation="horizontal", fraction=0.03, pad=0.05, aspect=60, ax=ax
    )

    # Top middle panel - observed map of sea ice
    ax = plt.subplot(2, 3, 2, projection=proj)
    plotdata = (regridded["ac"][month_index] * 100.0).to_masked_array()
    plotdata = np.ma.masked_where(valid_mask, plotdata)
    cb2 = _plot_map_panel(ax, x, y, plotdata, extent=extent)
    ax.set_title(f"NSIDC - Years {obs.time[0]} to {obs.time[1]}")
    fig.colorbar(
        cb2, orientation="horizontal", fraction=0.03, pad=0.05, aspect=60, ax=ax
    )

    # Top right panel - model minus observed difference
    ax = plt.subplot(2, 3, 3, projection=proj)
    _mod = np.where(
        np.isnan(model["ac"][month_index].data), 0.0, model["ac"][month_index].data
    )
    _obs = np.where(
        np.isnan(regridded["ac"][month_index].data),
        0.0,
        regridded["ac"][month_index].data,
    )
    plotdata = (_mod - _obs) * 100.0
    plotdata = np.ma.masked_where(valid_mask, plotdata)
    cb3 = _plot_map_panel(
        ax,
        x,
        y,
        plotdata,
        cmap="RdBu",
        vmin=-100.0,
        vmax=100.0,
        contour=False,
        extent=extent,
    )
    ax.set_title("Difference")
    fig.colorbar(
        cb3, orientation="horizontal", fraction=0.03, pad=0.05, aspect=60, ax=ax
    )

    # Bottom left panel - annual cycle of sea ice area
    ax = plt.subplot(2, 3, 4)
    _plot_annual_cycle(ax, model["area"], obs["area"], roll=month_index - 9)
    ax.set_title("Sea Ice Area")
    ax.set_ylabel("1.e6 km^2")

    # Bottom middle panel - annual cycle of sea ice extent
    ax = plt.subplot(2, 3, 5)
    _plot_annual_cycle(ax, model["ext"], obs["ext"], roll=month_index - 9)
    ax.set_title("Sea Ice Extent")
    ax.set_ylabel("1.e6 km^2")

    # Text statistics annotations
    fig.text(0.67, 0.39, "Annual Sea Ice Area", fontsize=10)
    fig.text(0.67, 0.375, "Model Max: %0.5f" % (model["area"].max()), fontsize=10)
    fig.text(0.67, 0.36, "Obs Max: %0.5f" % (obs["area"].max()), fontsize=10)
    fig.text(0.67, 0.345, "Model Min: %0.5f" % (model["area"].min()), fontsize=10)
    fig.text(0.67, 0.33, "Obs Min: %0.5f" % (obs["area"].min()), fontsize=10)
    fig.text(0.67, 0.285, "Annual Sea Ice Extent", fontsize=10)
    fig.text(0.67, 0.27, "Model Max: %0.5f" % (model["ext"].max()), fontsize=10)
    fig.text(0.67, 0.255, "Obs Max: %0.5f" % (obs["ext"].max()), fontsize=10)
    fig.text(0.67, 0.24, "Model Min: %0.5f" % (model["ext"].min()), fontsize=10)
    fig.text(0.67, 0.225, "Obs Min: %0.5f" % (obs["ext"].min()), fontsize=10)

    # Top header title
    plt.subplots_adjust(top=0.8)
    if region == "nh":
        fig.text(
            0.5,
            0.92,
            "Northern Hemisphere Sea Ice",
            ha="center",
            fontsize=22,
            weight="bold",
        )
    elif region == "sh":
        fig.text(
            0.5,
            0.92,
            "Southern Hemisphere Sea Ice",
            ha="center",
            fontsize=22,
            weight="bold",
        )
    fig.text(
        0.5,
        0.89,
        f"Climatological {month} Sea Ice Concentration  /  "
        + "Annual Cycle of Area and Extent",
        ha="center",
        fontsize=14,
    )
    if label is not None:
        fig.text(0.5, 0.86, label, ha="center", fontsize=14)

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence"""

    ## parameters
    # interactive = args.interactive
    # outdir = args.outdir
    # pltfmt = args.format
    # infile = args.infile
    # static = args.static
    # month = args.month
    # obsfile = args.obsfile
    # region = args.region
    # label = args.label

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # --- the main show ---
    ds, dobs, valid_mask = read(dictArgs)

    current_dir = os.getcwd()
    tmpdir = tempfile.mkdtemp()
    os.chdir(tmpdir)
    model, obs, regridded = calculate(ds, dobs, region=dictArgs["region"])
    os.chdir(current_dir)
    shutil.rmtree(tmpdir)

    fig = plot(
        model,
        obs,
        regridded,
        valid_mask,
        label=dictArgs["label"],
        region=dictArgs["region"],
        month=dictArgs["month"],
    )
    # ---------------------

    filename = f"{dictArgs['outdir']}/seaice.{dictArgs['region']}"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    """Function to make compatibile with the superwrapper"""
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



from .. import generic_yz_annual_bias_1x1deg as generic

from omlabs.m6plot.formatting import pmCI, linCI

possible_variable_names = ["so", "salt", "salinity", "SALT", "SALINITY"]


def parse_and_run(cliargs=None):
    dictArgs = parse(cliargs)
    imgbufs = run(dictArgs)
    return imgbufs


def parse(cliargs=None, template=False):
    if template is True:
        dictArgs = generic.parse(template=True)
    else:
        cmdLineArgs = generic.parse(cliargs)
        dictArgs = vars(cmdLineArgs)  # convert parser args to dict
    # add custom option for data read
    dictArgs["possible_variable_names"] = possible_variable_names
    dictArgs["surface_default_depth"] = 2.5
    # add custom options for plot
    dictArgs["var"] = "so"
    dictArgs["units"] = "[ppt]"
    dictArgs["clim_diff"] = pmCI(0.125, 2.25, 0.25)
    dictArgs["clim_compare"] = linCI(20, 30, 10, 31, 39, 0.5)
    dictArgs["cmap_diff"] = "dunnePM"
    dictArgs["cmap_compare"] = "dunneRainbow"
    return dictArgs


def run(dictArgs):
    imgbufs = generic.run(dictArgs)
    return imgbufs



# -*- coding: utf-8 -*-
"""m6plot.coords - routines to handle array operations associated
with 2-D grids
"""

__all__ = [
    "boundaryStats",
    "expand",
    "expandI",
    "expandJ",
    "section2quadmesh",
]

for func in __all__:
    exec(f"from .{func} import *")



import os
import warnings

_diag_dir = os.path.split(os.path.abspath(__file__))[0]
_exclude_list = ["__pycache__"]

_diaglist = [_x for _x in os.listdir(_diag_dir)]
_diaglist = [f"{_diag_dir}/{_x}" for _x in _diaglist]
_diaglist = [_x for _x in _diaglist if os.path.isdir(_x) and os.path.basename(_x) not in _exclude_list]
_diaglist = sorted([os.path.basename(_x) for _x in _diaglist])

for _diag in _diaglist:
    try:
        exec(f"from . import {_diag}")
    except Exception as exc:
        warnings.warn(f"Unable to import diagnostic: {_diag}")

del os
del warnings
del _diag
del _diaglist
del _diag_dir
del _exclude_list


#!/usr/bin/env python3

import argparse
import pkg_resources as pkgr
import io
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from omlabs import m6plot
import palettable
import xarray as xr
import warnings

from xwmt.preprocessing import preprocessing
from xwmt.swmt import swmt

from omlabs.om4common import horizontal_grid
from omlabs.om4common import image_handler
from omlabs.om4common import date_range
from omlabs.om4common import open_intake_catalog
from omlabs.om4parser import default_diag_parser

warnings.filterwarnings("ignore", message=".*csr_matrix.*")
warnings.filterwarnings("ignore", message=".*dates out of range.*")


def calculate(ds, bins, group_tend):
    """Calculates watermass transformation from surface fluxes"""

    G = swmt(ds).G("sigma0", bins=bins, group_tend=group_tend)

    # If tendencies were grouped then G is a DataArray
    # For consistency in plotting function, convert it to a dataset
    if group_tend:
        G = G.to_dataset()

    return G


def parse(cliargs=None, template=False):
    """
    Function to capture the user-specified command line options
    """
    description = """ """

    parser = default_diag_parser(
        description=description,
        template=template,
        exclude=["obsfile", "topog", "config", "platform", "basin"],
    )

    parser.add_argument(
        "--bins",
        type=str,
        default="20,30,0.1",
        help="Density bins at which to evaluate transformation, provided as start, stop, increment.",
    )

    parser.add_argument(
        "--group_tend",
        dest="group_tend",
        action="store_true",
        help="Group heat and salt tendencies together, i.e. only return the total transformation. Not passing this could lead to a performance cost.",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(
    dictArgs,
    heatflux_varname="hfds",
    saltflux_varname="sfdsi",
    fwflux_varname="wfo",
    sst_varname="tos",
    sss_varname="sos",
):
    """Read in surface flux data"""

    infile = dictArgs["infile"]
    ds = xr.open_mfdataset(infile, combine="by_coords", use_cftime=True)

    ### NEED TO IMPOSE CHECK TO MAKE SURE THIS IS NOT ANNUAL DATA

    # Check that all required variables are here
    check_vars = [
        heatflux_varname,
        saltflux_varname,
        fwflux_varname,
        sst_varname,
        sss_varname,
    ]
    check = all(item in ds.data_vars for item in check_vars)
    if not check:
        missing = set(check_vars) - set(ds.data_vars)
        raise RuntimeError(
            "Necessary variable {} not present in dataset".format(missing)
        )

    ds["areacello"] = xr.open_mfdataset(dictArgs["static"])["areacello"]
    ds["deptho"] = xr.open_mfdataset(dictArgs["static"])["deptho"]
    ds["geolat"] = xr.open_mfdataset(dictArgs["static"])["geolat"]
    ds["geolon"] = xr.open_mfdataset(dictArgs["static"])["geolon"]

    ### WMT preprocessing step
    # Perhaps we should pull out some of what happens in here ?
    ds = preprocessing(ds, grid=ds, decode_times=False, verbose=False)

    if "bins" in dictArgs:
        bins_args = dictArgs["bins"]
        bins_args = tuple([float(x) for x in bins_args.split(",")])
        bins = np.arange(*bins_args)
    else:
        # Default bins
        bins = np.arange(20, 30, 0.1)

    # Retrieve group_tend boolean
    group_tend = dictArgs["group_tend"]

    return (ds, bins, group_tend)


def plot(G):

    # Don't plot first or last bin (expanded to capture full range)
    G = G.isel(sigma0=slice(1, -1))
    levs = G["sigma0"].values

    # Take annual mean and load
    G = G.mean("time").load()
    # Get terms in dataset
    terms = list(G.data_vars)

    fig, ax = plt.subplots()
    # Plot each term
    for term in terms:
        if term == "heat":
            color = "tab:red"
        elif term == "salt":
            color = "tab:blue"
        else:
            color = "k"
        ax.plot(levs, G[term], label=term, color=color)

    # If terms were not grouped then sum them up to get total
    if len(terms) > 1:
        total = xr.zeros_like(G[terms[0]])
        for term in terms:
            total += G[term]
        ax.plot(levs, total, label="total", color="k")

    ax.legend()
    ax.set_xlabel("SIGMA0")
    ax.set_ylabel("TRANSFORMATION ($m^3s^{-1}$)")
    ax.autoscale(enable=True, axis="x", tight=True)

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence"""

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # --- the main show ---
    (ds, bins, group_tend) = read(dictArgs)

    G = calculate(ds, bins, group_tend)

    fig = plot(G)

    filename = f"{dictArgs['outdir']}/surface_wmt"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



from .heat_transport import parse, read, calculate, plot, run, parse_and_run

__description__ = "Plots global and basin-average poleward heat transport"
__ppstreams__ = [
    "ocean_monthly/av",
]
__ppvars__ = ["T_ady_2d"]



from . import cm
from . import coords
from . import formatting
from . import stats
from . import addStatusBar
from . import addInteractiveCallbacks

import matplotlib.pyplot as plt
import numpy as np


def yzplot(
    field,
    y=None,
    z=None,
    area=None,
    depth=None,
    ylabel=None,
    yunits=None,
    zlabel=None,
    zunits=None,
    splitscale=None,
    title="",
    suptitle="",
    clim=None,
    colormap=None,
    extend=None,
    centerlabels=False,
    nbins=None,
    landcolor=[0.5, 0.5, 0.5],
    aspect=[16, 9],
    resolution=576,
    axis=None,
    ignore=None,
    save=None,
    debug=False,
    show=False,
    interactive=False,
):
    """
    Renders section plot of scalar field, field(x,z).

    Arguments:
    field       Scalar 2D array to be plotted.
    y           y (or x) coordinate (1D array). If y is the same size as field then x is treated as
                the cell center coordinates.
    z           z coordinate (1D or 2D array). If z is the same size as field then y is treated as
                the cell center coordinates.
    ylabel      The label for the x axis. Default 'Latitude'.
    yunits      The units for the x axis. Default 'degrees N'.
    zlabel      The label for the z axis. Default 'Elevation'.
    zunits      The units for the z axis. Default 'm'.
    splitscale    A list of depths to define equal regions of projection in the vertical, e.g. [0.,-1000,-6500]
    title       The title to place at the top of the panel. Default ''.
    suptitle    The super-title to place at the top of the figure. Default ''.
    clim        A tuple of (min,max) color range OR a list of contour levels. Default None.
    colormap    The name of the colormap to use. Default None.
    extend      Can be one of 'both', 'neither', 'max', 'min'. Default None.
    centerlabels If True, will move the colorbar labels to the middle of the interval. Default False.
    nbins       The number of colors levels (used is clim is missing or only specifies the color range).
    landcolor   An rgb tuple to use for the color of land (no data). Default [.5,.5,.5].
    aspect      The aspect ratio of the figure, given as a tuple (W,H). Default [16,9].
    resolution  The vertical resolution of the figure given in pixels. Default 720.
    axis         The axis handle to plot to. Default None.
    ignore      A value to use as no-data (NaN). Default None.
    save        Name of file to save figure in. Default None.
    debug       If true, report stuff for debugging. Default False.
    show        If true, causes the figure to appear on screen. Used for testing. Default False.
    interactive If true, adds interactive features such as zoom, close and cursor. Default False.
    """

    # set visual backend
    if interactive is False:
        plt.switch_backend("Agg")

    c = cm.dunne_pm()
    c = cm.dunne_rainbow()

    _z = z.copy()

    # Create coordinates if not provided
    ylabel, yunits, zlabel, zunits = formatting.createYZlabels(
        y, z, ylabel, yunits, zlabel, zunits
    )
    if debug:
        print("y,z label/units=", ylabel, yunits, zlabel, zunits)

    # Check coordinate dimensions
    if len(y.shape) == 1 and len(z.shape) == 1:
        assert (len(z), len(y)) == field.shape
    elif len(y) == z.shape[-1]:
        y = coords.expand(y)
    elif len(y) == z.shape[-1] + 1:
        y = y
    else:
        raise Exception(
            "Length of y coordinate should be equal or 1 longer than horizontal length of z"
        )

    # Ignore a user-defined value if provided
    if ignore is not None:
        maskedField = np.ma.masked_array(field, mask=[field == ignore])
    else:
        maskedField = field.copy()

    if len(y.shape) == 1 and len(z.shape) == 1:
        # Use raw coordinate and array data for plotting
        yCoord = y
        zCoord = z
        field2 = maskedField

        # Compute interfaces that are used for generating weights in the
        # stats routines
        y = coords.expand(y)
        interfaces = [0.0]
        for n in range(0, len(z)):
            interfaces.append(z[n] - np.abs((z[n] - interfaces[n])))
        z = np.array(interfaces)
        z = np.tile(z[:, None], (1, len(y) - 1))

    else:
        # Do vertical coordinate transformation on native output
        yCoord, zCoord, field2 = coords.section2quadmesh(y, z, maskedField)
        yLims = np.amin(yCoord), np.amax(yCoord)
        zLims = coords.boundaryStats(zCoord)

    # Diagnose statistics
    sMin, sMax, sMean, sStd, sRMS = stats.calc(
        maskedField, stats.yzWeight(y, z), debug=debug
    )

    # Choose colormap
    if nbins is None and (clim is None or len(clim) == 2):
        nbins = 35
    if colormap is None:
        colormap = cm.chooseColorMap(sMin, sMax)
    cmap, norm, extend = cm.chooseColorLevels(
        sMin, sMax, colormap, clim=clim, nbins=nbins, extend=extend
    )

    # Create a topography mask
    if depth is not None:
        depth = np.tile(depth[None, :, :], (len(_z), 1, 1))
        depth = depth.min(axis=-1)
        ztile = np.tile(_z[:, None], (1, depth.shape[-1]))
        topomask = np.where(np.less(ztile, depth), 0.0, 1.0)
        topomask = np.ma.masked_where(np.equal(topomask, 0.0), topomask)
    else:
        topomask = 1.0

    field2 = field2 * topomask

    if axis is None:
        formatting.setFigureSize(aspect, resolution, debug=debug)
        # plt.gcf().subplots_adjust(left=.10, right=.99, wspace=0, bottom=.09, top=.9, hspace=0)
        axis = plt.gca()

    plt.pcolormesh(yCoord, zCoord, field2, cmap=cmap, norm=norm, shading="auto")
    if interactive:
        addStatusBar(yCoord, zCoord, field2)
    cb = plt.colorbar(fraction=0.08, pad=0.02, extend=extend)
    if centerlabels and len(clim) > 2:
        cb.set_ticks(0.5 * (clim[:-1] + clim[1:]))
    axis.set_facecolor(landcolor)
    if splitscale is not None:
        for zzz in splitscale[1:-1]:
            plt.axhline(zzz, color="k", linestyle="--")
        axis.set_yscale("splitscale", zval=splitscale)
        axis.invert_yaxis()
    # plt.xlim(yLims)
    # plt.ylim(zLims)
    axis.annotate(
        "max=%.5g\nmin=%.5g" % (sMax, sMin),
        xy=(0.0, 1.01),
        xycoords="axes fraction",
        verticalalignment="bottom",
        fontsize=10,
    )
    if sMean is not None:
        axis.annotate(
            "mean=%.5g\nrms=%.5g" % (sMean, sRMS),
            xy=(1.0, 1.01),
            xycoords="axes fraction",
            verticalalignment="bottom",
            horizontalalignment="right",
            fontsize=10,
        )
        axis.annotate(
            " sd=%.5g\n" % (sStd),
            xy=(1.0, 1.01),
            xycoords="axes fraction",
            verticalalignment="bottom",
            horizontalalignment="left",
            fontsize=10,
        )
    if len(ylabel + yunits) > 0:
        plt.xlabel(formatting.label(ylabel, yunits))
    if len(zlabel + zunits) > 0:
        plt.ylabel(formatting.label(zlabel, zunits))
    if len(title) > 0:
        plt.title(title)
    if len(suptitle) > 0:
        plt.suptitle(suptitle)

    if save is not None:
        plt.savefig(save)
    if interactive:
        addInteractiveCallbacks()
    if show:
        plt.show(block=False)

    return plt.gcf()



import importlib.metadata as ilm

msg = ilm.metadata("omlabs")

__name__ = msg["Name"]
__version__ = msg["Version"]
__license__ = msg["License"]
__email__ = msg["Maintainer-email"]
__description__ = msg["Summary"]
__requires__ = msg["Requires-Dist"]
__requires_python__ = msg["Requires-Python"]

from . import diags
from . import m6plot
from . import om4parser
from . import m6toolbox



def brownblue_cmap():
    # The color map below is from the Light & Bartlein collection
    # which is tested for several different forms of colorblindness
    #
    # Reference:
    # A. Light & P.J. Bartlein, "The End of the Rainbow? Color Schemes for
    # Improved Data Graphics," Eos,Vol. 85, No. 40, 5 October 2004.
    # http://geog.uoregon.edu/datagraphics/EOS/Light-and-Bartlein.pdf

    lb_brownblue_values = numpy.array(
        [
            [144, 100, 44],
            [187, 120, 54],
            [225, 146, 65],
            [248, 184, 139],
            [244, 218, 200],
            [255, 255, 255],  # [241,244,245],
            [207, 226, 240],
            [160, 190, 225],
            [109, 153, 206],
            [70, 99, 174],
            [24, 79, 162],
        ]
    )

    lb_brownblue_values = lb_brownblue_values / 255.0
    lb_brownblue_values = lb_brownblue_values[::-1, :]

    import matplotlib

    cmap = matplotlib.colors.LinearSegmentedColormap.from_list(
        "brownblue", lb_brownblue_values
    )
    cmap.set_bad("w")
    matplotlib.cm.register_cmap(cmap=cmap)
    return cmap



#!/usr/bin/env python3

""" Generic Drift Plot Diagnostic """

import matplotlib.pyplot as plt
import numpy as np
import xarray as xr

from omlabs.om4parser import default_diag_parser
from omlabs.om4common import image_handler


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments
    """

    description = "Generic diagnostic to plot a depth-vs-time drift time series"

    parser = default_diag_parser(
        description=description,
        template=template,
        exclude=[
            "suptitle",
            "gridspec",
            "static",
            "basin",
            "obsfile",
            "hgrid",
            "topog",
        ],
    )

    parser.add_argument(
        "--description",
        type=str,
        required=False,
        default="Layer-average drift vs. time",
        help="description string in subtitle",
    )
    parser.add_argument(
        "--varname",
        type=str,
        required=False,
        default=None,
        help="variable name to plot",
    )
    parser.add_argument(
        "--range",
        type=float,
        required=False,
        default=1.0,
        help="min/max data range for contouring",
    )
    parser.add_argument(
        "--interval", type=float, required=False, default=0.1, help="contour interval"
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs):
    """Read required fields

    Parameters
    ----------
    dictArgs : dict
        Dictionary containing argparse options

    Returns
    -------
    xarray.DataSet
    """

    dset = xr.open_mfdataset(dictArgs["infile"], use_cftime=True)

    return dset


def calculate(dset, varname, tcoord="time"):
    """Function to calculate the drift time series

    This function calculates the drift of a given variable contained
    in a dataset as the anomaly relative to the first time level

    Parameters
    ----------
    dset : xarray.Dataset
        Input dataset
    varname : str
        Variable name to be analyzed
    tcoord : str, optional
        Time coordinate variable name, by default "time"

    Returns
    -------
    xarray.DataArray
        Anomaly time series
    """

    return dset[varname] - dset[varname].isel({tcoord: 0})


def plot(
    var,
    time=None,
    depth=None,
    tcoord="time",
    zcoord="z_l",
    label=None,
    vardesc=None,
    rangemax=1.0,
    interval=0.1,
    splitscale=2000.0,
):
    """Function to plot a depth vs. time contour figure

    Parameters
    ----------
    var : xarray.DataArray or np.Array
        Input data array in either xarray or numpy format
    time : np.Array, optional
        Time coordiate array in NumPy format, otherwise the array is
        obtained from the xarray variable coordinates, by default None
    depth : np.Array, optional
        Depth coordiate array in NumPy format, otherwise the array is
        obtained from the xarray variable coordinates, by default None
    tcoord : str, optional
        Name of xarray time coordinate, by default "time"
    zcoord : str, optional
        Name of xarray depth coordinate, by default "z_l"
    label : str, optional
        Experiment name label, by default None
    vardesc : str, optional
        Variable description string (subtitle), by default None
    rangemax : float, optional
        Plotting range defined by +/- `rangemax`, by default 1.
    interval : float, optional
        Contour interval, by default 0.1
    splitscale : float, optional
        Depth level that defines split vertical scale boundary,
        by default 2000.

    Returns
    -------
    matplotlib.figure.Figure
        Figure handle
    """

    # get NumPy arrays from xarray
    if isinstance(var, xr.DataArray):
        time = var[tcoord].values
        depth = var[zcoord].values
        var = var.values

    # setup figure and axis
    fig = plt.figure(figsize=(6, 3))
    ax = plt.subplot(1, 1, 1)

    # setup contour range and intervals
    levels = np.arange(-1.0 * rangemax, rangemax, interval)
    cb = plt.contourf(time, depth, var.T, levels=levels, cmap="RdBu_r", extend="both")

    # add line contours and labels
    cs = plt.contour(time, depth, var.T, levels=levels, colors=["k"], linewidths=0.5)
    ax.clabel(cs, fontsize=6, rightside_up=True)

    # add vertical line at split-scale location
    _ = plt.axhline(splitscale, color="k", linestyle="dashed", linewidth=0.75)
    ax.set_yscale("splitscale", zval=[6500.0, splitscale, 0.0])

    # add colorbar
    cb = plt.colorbar(cb)

    # set tick label sizes
    ax.tick_params(labelsize=8)
    cb.ax.tick_params(labelsize=8)

    # add titles
    ax.text(0.0, 1.1, label, transform=ax.transAxes, weight="bold", fontsize=8)
    ax.text(0.0, 1.04, vardesc, transform=ax.transAxes, style="italic", fontsize=7)

    # add min/max values
    maxval = str(round(var.max(), 2))
    minval = str(round(var.min(), 2))
    ax.text(
        1.0,
        1.04,
        f"Min: {minval}",
        ha="right",
        transform=ax.transAxes,
        style="italic",
        fontsize=7,
    )
    ax.text(
        1.0,
        1.09,
        f"Max: {maxval}",
        ha="right",
        transform=ax.transAxes,
        style="italic",
        fontsize=7,
    )

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence

    Parameters
    ----------
    dictArgs : dict
        Dictionary of parsed options

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # read in data
    dset = read(dictArgs)

    # calculate
    arr = calculate(dset, dictArgs["varname"])

    # make the plots
    fig = plot(
        arr,
        label=dictArgs["label"],
        vardesc=dictArgs["description"],
        rangemax=dictArgs["range"],
        interval=dictArgs["interval"],
    )

    filename = f"{dictArgs['outdir']}/drift_{dictArgs['varname']}"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Parameters
    ----------
    cliargs : argparse, optional
        command line arguments from upstream instance, by default None

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



# -*- coding: utf-8 -*-
"""m6plot.cm - routines to handle colormaps used by matplotlib
"""

__version__ = "0.0.0"
__all__ = [
    "chooseColorLevels",
    "chooseColorMap",
    "dunne_pm",
    "dunne_rainbow",
]

for func in __all__:
    exec(f"from .{func} import *")



def test_default_diag_parser():
    from omlabs.om4parser import default_diag_parser
    from omlabs.om4parser import DefaultDictParser

    parser = default_diag_parser(template=True)
    args = parser.parse_args([])
    args = args.__dict__
    num_default_options = len(list(args.keys()))
    assert isinstance(args, dict), "unable to obtain dict of options"
    assert num_default_options > 0, "no options are present"

    parser = default_diag_parser(template=False)
    args = parser.parse_args(["foo"])
    _num_keys = len(list(args.__dict__.keys()))
    assert (
        _num_keys == num_default_options
    ), "argparse object does not have the same number of keys"

    parser = default_diag_parser(template=False, exclude="gridspec")
    args = parser.parse_args(["foo"])
    _num_keys = len(list(args.__dict__.keys()))
    assert _num_keys == (num_default_options - 1), "excluding a single string failed"

    parser = default_diag_parser(template=False, exclude=["gridspec", "basin"])
    args = parser.parse_args(["foo"])
    _num_keys = len(list(args.__dict__.keys()))
    assert _num_keys == (num_default_options - 2), "excluding a list failed"



import warnings
import palettable

import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

from omlabs.m6plot.formatting import VerticalSplitScale


def plot_rho_panel(ax, arr, lat, rho, levels=None, title=None):
    """Function to individual panel of overturning streamfunction in sigma space

    Parameters
    ----------
    ax : matplotlib.axes.axis
        axis handle to use for plotting
    arr : numpy.ma.MaskedArray
        Input array with dimensions (basin,rho,lat)
    lat : numpy.array
        1-D array of latitude values
    rho : numpy.array
        1-D array of rho values
    levels : list, optional
        contour intervals, by default None
    title : str, optional
        panel title at the top left corner, by default None

    Returns
    -------
    matplotlib.collections.QuadMesh
        output from pcolormesh
    """

    # contour levels
    if levels is None:
        levels = np.arange(-40, 45, 5)

    cb = ax.contourf(lat, rho, arr, levels=levels, cmap="RdBu_r")
    cs = ax.contour(lat, rho, arr, levels=levels, colors=["k"], linewidths=0.5)

    # formatting vertical axis
    ax.set_yscale("splitscale", zval=[1037.2, 1036.5, 1028])

    # set labels
    ax.set_ylabel("sigma2")
    ax.text(0.01, 1.02, title, ha="left", fontsize=10, transform=ax.transAxes)

    return cb


def plot_rho(otsfn, lat, rho, label=None, dates=None):
    """

    Parameters
    ----------
    otsfn : numpy.ma.MaskedArray
        Input array with dimensions (basin,rho,lat)
    rho : numpy.array
        1-D array of rho values
    depth : numpy.array
        1-D array of depth values
    label : str, optional
        Experiment name / top line label, by default None
    dates : tuple, optional
        Tuple of date ranges, by default None

    Returns
    -------
    matplotlib.figure.Figure
        Figure handle
    """

    # setup figure handle
    fig = plt.figure(figsize=(8.5, 11))

    # Panel 1 (top): Atlantic-Arctic
    ax1 = plt.subplot(3, 1, 1)
    levels = np.arange(-42, 45, 3)
    title = "a. Atlantic-Arctic MOC [Sv]"
    cb1 = plot_rho_panel(ax1, otsfn[0], lat, rho, levels=levels, title=title)

    # Panel 2 (middle): Indo-Pacific
    ax2 = plt.subplot(3, 1, 2)
    title = "b. Indo-Pacific MOC [Sv]"
    cb2 = plot_rho_panel(ax2, otsfn[1], lat, rho, levels=levels, title=title)

    # Panel 3 (bottom): Global
    ax3 = plt.subplot(3, 1, 3)
    title = "c. Global MOC [Sv]"
    cb3 = plot_rho_panel(ax3, otsfn[2], lat, rho, levels=levels, title=title)

    plt.suptitle(label)
    # format date string
    if dates is not None:
        dates = f"Years {dates[0]} - {dates[1]}"
        ax1.text(0.98, 1.02, dates, ha="right", transform=ax1.transAxes)

    # colorbar
    cbar_ax = fig.add_axes([0.30, 0.05, 0.4, 0.01])
    fig.colorbar(cb1, cax=cbar_ax, orientation="horizontal")

    return fig



import numpy as np
from .linCI import *


def pmCI(min, max, ci, *args):
    """
    Returns list of linearly spaced contour intervals from -max to -min then min to max with spacing ci.
    Unline np.arange this max is included IF max = min + ci*N for an integer N.
    """
    ci = linCI(min, max, ci, *args)
    if ci[0] > 0:
        return np.concatenate((-ci[::-1], ci))
    else:
        return np.concatenate((-ci[::-1], ci[1:]))



# -*- coding: utf-8 -*-
"""m6plot.formatting - routines to customize the format of the plots
"""

__all__ = [
    "createXYcoords",
    "createXYlabels",
    "createYZlabels",
    "label",
    "linCI",
    "pmCI",
    "setFigureSize",
    "VerticalSplitScale",
]

for func in __all__:
    exec(f"from .{func} import *")



def plotBasemapPanel(
    field, sector, xCoord, yCoord, lonRange, latRange, cmap, norm, interactive, extend
):
    if sector == "arctic":
        m = Basemap(projection="npstere", boundinglat=60, lon_0=-120, resolution="l")
    elif sector == "shACC":
        m = Basemap(projection="spstere", boundinglat=-45, lon_0=-120, resolution="l")
    else:
        m = Basemap(
            projection="mill",
            lon_0=-120.0,
            resolution="l",
            llcrnrlon=lonRange[0],
            llcrnrlat=latRange[0],
            urcrnrlon=lonRange[1],
            urcrnrlat=latRange[1],
        )
    m.drawmapboundary(fill_color="0.85")
    im0 = m.pcolormesh(
        numpy.minimum(xCoord, 60.0),
        yCoord,
        (field),
        shading="flat",
        cmap=cmap,
        norm=norm,
        latlon=True,
    )
    m.drawcoastlines()
    if interactive:
        addStatusBar(xCoord, yCoord, field)
    if extend is None:
        extend = extend
    cb1 = m.colorbar(pad=0.15, extend=extend)
    if sector == "tropPac":
        drawNinoBoxes(m)



import intake
import pkg_resources


def test_catalogs_are_present():
    f = pkg_resources.resource_filename("omlabs", "catalogs/obs_catalog_gfdl.yml")
    cat = intake.open_catalog(f)
    assert isinstance(cat, intake.catalog.local.YAMLFileCatalog)



def createTZlabels(t, z, tlabel, tunits, zlabel, zunits):
    """
    Checks that y and z labels are appropriate and tries to make some if they are not.
    """
    if t is None:
        if tlabel is None:
            tlabel = "t"
        if tunits is None:
            tunits = ""
    else:
        if tlabel is None:
            tlabel = "Time"
        if tunits is None:
            tunits = ""
    if z is None:
        if zlabel is None:
            zlabel = "k"
        if zunits is None:
            zunits = ""
    else:
        if zlabel is None:
            zlabel = "Elevation"
        if zunits is None:
            zunits = "m"
    return tlabel, tunits, zlabel, zunits



#!/usr/bin/env python3

import argparse
import pkg_resources as pkgr
import intake
import io
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from omlabs import m6plot
import palettable
import xarray as xr
import xoverturning
import warnings

from omlabs.om4common import horizontal_grid
from omlabs.om4common import read_topography
from omlabs.om4common import image_handler
from omlabs.om4common import is_symmetric
from omlabs.om4common import generate_basin_masks
from omlabs.om4common import date_range
from omlabs.om4parser import default_diag_parser

from omlabs.diags.moc.plotting import plot_z
from omlabs.diags.moc.plotting import plot_rho

warnings.filterwarnings("ignore", message=".*csr_matrix.*")
warnings.filterwarnings("ignore", message=".*dates out of range.*")


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments
    """

    description = " "

    parser = default_diag_parser(description=description, template=template)

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs, vcomp="vmo", ucomp="umo"):
    """Read required fields to plot MOC in omlabs

    Parameters
    ----------
    dictArgs : dict
        Dictionary containing argparse options
    vcomp : str, optional
        Name of meridional component of total residual
        freshwater transport, by default "vmo"
    ucomp : str, optional
        Name of zonal component of total residual
        freshwater transport, by default "umo"

    Returns
    -------
    xarray.DataSet
        Two xarray datasets; one containing `umo`, `vmo`
        and the other containing the grid information
    """

    # initialize an xarray.Dataset to hold the output
    dset = xr.Dataset()
    dset_grid = xr.Dataset()

    # read the infile and get u, v transport components
    infile = dictArgs["infile"]
    ds = xr.open_mfdataset(infile, combine="by_coords")
    dset["umo"] = ds[ucomp]
    dset["vmo"] = ds[vcomp]

    # detect symmetric grid
    outputgrid = "symetric" if is_symmetric(dset) else "nonsymetric"

    # determine vertical coordinate
    layer = "z_l" if "z_l" in ds.dims else "rho2_l" if "rho2_l" in ds.dims else None
    assert layer is not None, "Unrecognized vertical coordinate."

    # get vertical coordinate edges
    interface = "z_i" if layer == "z_l" else "rho2_i" if layer == "rho2_l" else None
    dset[interface] = ds[interface]

    # save layer and interface info for use later in the workflow
    dset.attrs["layer"] = layer
    dset.attrs["interface"] = interface

    # get horizontal v-cell grid info
    dsV = horizontal_grid(
        dictArgs, coords=ds.coords, point_type="v", outputgrid=outputgrid
    )
    dset_grid["geolon_v"] = xr.DataArray(dsV.geolon.values, dims=("yq", "xh"))
    dset_grid["geolat_v"] = xr.DataArray(dsV.geolat.values, dims=("yq", "xh"))
    dset_grid["wet_v"] = xr.DataArray(dsV.wet.values, dims=("yq", "xh"))

    # get topography info
    depth = read_topography(dictArgs, coords=ds.coords, point_type="t")
    depth = np.where(np.isnan(depth.to_masked_array()), 0.0, depth)
    dset_grid["deptho"] = xr.DataArray(depth, dims=("yh", "xh"))

    # dset_grid requires `xq`
    dset_grid["xq"] = dset.xq

    # save date range as an attribute
    dates = date_range(ds)
    dset.attrs["dates"] = dates

    return (dset, dset_grid)


def calculate(dset, dset_grid):
    """Main calculation function

    Parameters
    ----------
    dset : xarray.Dataset
        Input dataset with grid values, umo, and vmo

    Returns
    -------
    xarray.DataArray
        Time-mean overturning streamfunction by basin
    """

    layer = dset.layer
    interface = dset.interface

    # define list of basins
    basins = ["atl-arc", "indopac", "global"]

    # determine options to pass to xoverturning.calcmoc
    mask_output = True if layer == "z_l" else False
    vertical = "rho2" if layer == "rho2_l" else "z" if layer == "z_l" else None

    # iterate over basins
    otsfn = [
        xoverturning.calcmoc(
            dset,
            dset_grid,
            mask_output=mask_output,
            output_true_lat=True,
            basin=x,
            verbose=False,
            vertical=vertical,
        )
        for x in basins
    ]

    # combine into single DataArray
    otsfn = xr.concat(otsfn, dim="basin")
    otsfn = otsfn.transpose(*("time", "basin", ...))

    # take the time mean
    otsfn = otsfn.squeeze()
    if "time" in otsfn.dims:
        if len(otsfn["time"]) > 1:
            warnings.warn("performing non-weighted time average")
        otsfn = otsfn.mean(dim="time")

    return otsfn


def plot(dset, otsfn, label=None):
    """Plotting wrapper that redirects to either the z-level
    or sigma plotting routine based on the data type

    Parameters
    ----------
    dset : xarray.Dataset
        Input dataset with grid values, umo, and vmo
    otsfn : xarray.DataArray
        Time-mean overturning streamfunction by basin
    label : str, optional
        title/experiment name for plot labels, by default None

    Returns
    -------
    matplotlib.figure.Figure
        Figure handle
    """
    layer = dset.layer

    # z-coordinate plot
    if layer == "z_l":
        fig = plot_z(
            otsfn.values,
            otsfn.lat.values,
            otsfn.z_i.values,
            dates=dset.dates,
            label=label,
        )
    # sigma2-coordinate plot
    elif layer == "rho2_l":
        fig = plot_rho(
            otsfn.values,
            otsfn.lat.values,
            otsfn.rho2_i.values,
            dates=dset.dates,
            label=label,
        )

    else:
        raise ValueError(f"{layer} is not a recognized vertical coordinate.")

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence

    Parameters
    ----------
    dictArgs : dict
        Dictionary of parsed options

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # read in data
    dset, dset_grid = read(dictArgs)

    # calculate otsfn
    otsfn = calculate(dset, dset_grid)

    # make the plots
    fig = plot(dset, otsfn, dictArgs["label"])

    filename = f"{dictArgs['outdir']}/moc"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Parameters
    ----------
    cliargs : argparse, optional
        command line arguments from upstream instance, by default None

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



from .depth_time_temperature_drift import (
    parse,
    read,
    calculate,
    plot,
    run,
    parse_and_run,
)

__description__ = "Layer mean potential temperature drift vs. time"
__ppstreams__ = [
    "ocean_annual_z/ts",
]
__ppvars__ = ["thetao_xyave"]



def chooseColorMap(sMin, sMax, difference=None):
    """
    Based on the min/max extremes of the data, choose a colormap that fits the data.
    """
    if difference == True:
        return "dunnePM"
    elif sMin < 0 and sMax > 0:
        return "dunnePM"
    # elif sMax>0 and sMin<0.1*sMax: return 'hot'
    # elif sMin<0 and sMax>0.1*sMin: return 'hot_r'
    else:
        return "dunneRainbow"



import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import numpy as np
import cmocean
import xcompare
import xarray as xr
from xgcm import Grid

from xcompare import plot_three_panel
from omlabs.om4common import image_handler
from omlabs.om4common import open_intake_catalog
from omlabs.om4common import date_range
from omlabs.om4parser import default_diag_parser


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments

    example usage:
    omlabs stress_curl -s fname_static.nc --dataset OM4_windstress --period 1999-2018 fname_model_taux.nc fname_model_tauy.nc
    """

    description = " "

    exclude = ["basin", "hgrid", "topog", "gridspec", "config"]

    parser = default_diag_parser(
        description=description, template=template, exclude=exclude
    )

    parser.add_argument(
        "--dataset",
        type=str,
        required=False,
        default="OM4_windstress",
        help="Name of the observational dataset, \
              as provided in intake catalog",
    )

    parser.add_argument(
        "--period",
        type=str,
        required=False,
        default="1959-1978",
        help="Time period for OMIP2 dataset, available are 1959-1978 and 1999-2018",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs):
    ds_model = xr.open_mfdataset(dictArgs["infile"], use_cftime=True)
    dates = date_range(ds_model)

    if dictArgs["obsfile"] is not None:
        # priority to user-provided obs file
        dsobs = xr.open_mfdataset(
            dictArgs["obsfile"], combine="by_coords", decode_times=False
        )
    else:
        # use dataset from catalog, either from command line or default
        cat = open_intake_catalog(dictArgs["platform"], "OMIP2")
        obsdataset = f"{dictArgs['dataset']}_{dictArgs['period']}"
        ds_ref_tau = cat[obsdataset].to_dask()

    ds_static = xr.open_mfdataset(dictArgs["static"])

    # replace the nominal xq and yq by indices so that Xarray does not get confused.
    # Confusion arises since there are inconsistencies between static file grid and
    # model data grid for the last value of yq. We never need xq and yq for actual
    # calculations, so filling these arrays with any value is not going to change
    # any results. But Xarray needs them to be consistent between the two files when
    # doing the curl operation on the stress.
    ds_model["xq"] = xr.DataArray(np.arange(len(ds_model["xq"])), dims=["xq"])
    ds_model["yq"] = xr.DataArray(np.arange(len(ds_model["yq"])), dims=["yq"])
    ds_ref_tau["xq"] = xr.DataArray(np.arange(len(ds_ref_tau["xq"])), dims=["xq"])
    ds_ref_tau["yq"] = xr.DataArray(np.arange(len(ds_ref_tau["yq"])), dims=["yq"])
    ds_static["xq"] = xr.DataArray(np.arange(len(ds_static["xq"])), dims=["xq"])
    ds_static["yq"] = xr.DataArray(np.arange(len(ds_static["yq"])), dims=["yq"])

    ds_model.attrs = {"date_range": dates}

    return ds_model, ds_ref_tau, ds_static


def calc_curl_stress(
    ds,
    ds_static,
    varx="tauuo",
    vary="tauvo",
    areacello_bu="areacello_bu",
    xdim="lon",
    ydim="lat",
    rho0=1035.0,
    maskname="wet_c",
    lonname="geolon_c",
    latname="geolat_c",
):
    """Calculate curl of stress acting on surface of the ocean.

    Parameters
        ----------
    ds        : xarray.Dataset dataset with tauuo and tauvo
    ds_static : xarray.Dataset with grid values
    varname   : str, optional
        Name of the tauuo and tauvo variables, by default "tauuo" and "tauvo"
    area : str, optional
        Name of the area variable, by default "areacello_bu"
    xdim : str, optional
        Name of the longitude coordinate, by default "lon"
    ydim : str, optional
        Name of the latitude coordinate, by default "lat"
    rho0: float, optional
        Reference density of seawater, by default 1035.0 kg/m3
    maskname: str, optional
        Name of land/sea mask, by default wet_c
    lonname: str, optional
        Name of longitude variable, by default geolon_c
    latname: str, optional
        Name of latitude variable, by default geolat_c
    Returns
    -------
        xarray.DataArray stress_curl
        curl of surface ocean stress
    """

    area = ds_static[areacello_bu]
    taux = ds[varx].mean(dim="time")
    tauy = ds[vary].mean(dim="time")

    # fill nan with 0.0 since want 0.0 values over land for the curl operation
    taux = taux.fillna(0.0)
    tauy = tauy.fillna(0.0)

    grid = Grid(
        ds_static,
        coords={
            "X": {"center": "xh", "outer": "xq"},
            "Y": {"center": "yh", "outer": "yq"},
        },
        periodic=["X"],
    )

    stress_curl = -grid.diff(taux * ds_static.dxCu, "Y", boundary="fill") + grid.diff(
        tauy * ds_static.dyCv, "X", boundary="fill"
    )

    stress_curl = stress_curl / (area * rho0)
    stress_curl = stress_curl.where(ds_static[maskname] == 1)
    stress_curl = stress_curl.assign_coords(
        {lonname: ds_static[lonname], latname: ds_static[latname]}
    )

    return stress_curl


def calculate(ds_model, ds_ref_tau, ds_static, dictArgs):

    curl_model = xr.Dataset(
        {
            "stress_curl": calc_curl_stress(ds_model, ds_static),
            "areacello_bu": ds_static["areacello_bu"],
        }
    )

    curl_ref = xr.Dataset(
        {
            "stress_curl": calc_curl_stress(ds_ref_tau, ds_static),
            "areacello_bu": ds_static["areacello_bu"],
        }
    )

    results = xcompare.compare_datasets(curl_model, curl_ref)

    return results


def plot(dictArgs, results):

    fig = xcompare.plot_three_panel(
        results,
        "stress_curl",
        cmap=cmocean.cm.delta,
        projection=ccrs.Robinson(),
        coastlines=False,
        vmin=-3e-10,
        vmax=3e-10,
        diffvmin=-3e-10,
        diffvmax=3e-10,
        labels=[dictArgs["label"], "OMIP reference"],
        lon_range=(-180, 180),
        lat_range=(-90, 90),
    )
    return fig


def plot_old(
    field,
    vmin=-3e-10,
    vmax=3e-10,
    lat_lon_ext=[-180, 180, -85.0, 90.0],
    lon="geolon_c",
    lat="geolat_c",
    cmap=cmocean.cm.delta,
    title="Curl of stress (N/m**2) acting on ocean surface",
):

    # convert xarray to ordinary numpy arrays
    if isinstance(field, xr.DataArray):
        geolon = field[lon].values
        geolat = field[lat].values
        field = field.values

    fig = plt.figure(figsize=[22, 8])
    ax = fig.add_subplot(projection=ccrs.Robinson(), facecolor="grey")
    cb = ax.pcolormesh(
        geolon,
        geolat,
        field,
        transform=ccrs.PlateCarree(),
        vmin=vmin,
        vmax=vmax,
        cmap=cmap,
    )

    # add separate colorbar
    cb = plt.colorbar(cb, ax=ax, format="%.1e", extend="both", shrink=0.6)
    cb.ax.tick_params(labelsize=12)

    # add gridlines and extent of lat/lon
    ax.gridlines(color="black", alpha=0.5, linestyle="--")
    ax.set_extent(lat_lon_ext, crs=ccrs.PlateCarree())
    _ = plt.title(title, fontsize=14)

    return fig


def run(dictArgs):
    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    ds_model, ds_ref_tau, ds_static = read(dictArgs)

    results = calculate(ds_model, ds_ref_tau, ds_static, dictArgs)
    figs = plot(dictArgs, results)
    figs = [figs] if not isinstance(figs, list) else figs
    assert isinstance(figs, list), "Figures must be inside a list object"

    filenames = [
        f"{dictArgs['outdir']}/surface_stress_curl",
    ]

    imgbufs = image_handler(figs, dictArgs, filename=filenames)

    return imgbufs


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Returns
    -------
    io.BytesIO
    In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



from .surface_wmt import parse, read, calculate, plot, run, parse_and_run

__description__ = "Plots global watermass transformation (across density surfaces) from fluxes of heat, salt, and freshwater at the ocean surface"
__ppstreams__ = [
    "ocean_monthly/av",
]
__ppvars__ = ["hfds","wfo","sfdsi","tos","sos"]



from .depth_time_salinity_drift import parse, read, calculate, plot, run, parse_and_run

__description__ = "Layer mean practical salinity drift vs. time"
__ppstreams__ = [
    "ocean_annual_z/ts",
]
__ppvars__ = ["so_xyave"]



import numpy as np


def section2quadmesh(x, z, q, representation="pcm"):
    """
    Creates the appropriate quadmesh coordinates to plot a scalar q(1:nk,1:ni) at
    horizontal positions x(1:ni+1) and between interfaces at z(nk+1,ni), using
    various representations of the topography.
    Returns X(2*ni+1), Z(nk+1,2*ni+1) and Q(nk,2*ni) to be passed to pcolormesh.
    TBD: Optionally, x can be dimensioned as x(ni) in which case it will be extraplated as if it had
    had dimensions x(ni+1).

    Optional argument:

    representation='pcm' (default) yields a step-wise visualization, appropriate for
             z-coordinate models.
    representation='plm' yields a piecewise-linear visualization more representative
             of general-coordinate (and isopycnal) models.
    representation='linear' is the aesthetically most pleasing but does not
             represent the data conservatively.
    """

    if x.ndim != 1:
        raise Exception("The x argument must be a vector")
    if z.ndim != 2:
        raise Exception("The z argument should be a 2D array")
    if q.ndim != 2:
        raise Exception("The z argument should be a 2D array")
    qnk, qni = q.shape
    znk, zni = z.shape
    xni = x.size
    if zni != qni:
        raise Exception("The last dimension of z and q must be equal in length")
    if znk != qnk + 1:
        raise Exception(
            "The first dimension of z must be 1 longer than that of q. q has %i levels"
            % qnk
        )
    if xni != qni + 1:
        raise Exception("The length of x must 1 longer than the last dimension of q")

    if type(z) == np.ma.core.MaskedArray:
        z[z.mask] = 0
    if type(q) == np.ma.core.MaskedArray:
        qmin = np.amin(q)
        q[q.mask] = qmin

    periodicDomain = (
        abs((x[-1] - x[0]) - 360.0) < 1e-6
    )  # Detect if horizontal axis is a periodic domain

    if representation == "pcm":
        X = np.zeros((2 * qni))
        X[::2] = x[:-1]
        X[1::2] = x[1:]
        Z = np.zeros((qnk + 1, 2 * qni))
        Z[:, ::2] = z
        Z[:, 1::2] = z
        Q = np.zeros((qnk, 2 * qni - 1))
        Q[:, ::2] = q
        Q[:, 1::2] = (q[:, :-1] + q[:, 1:]) / 2.0
    elif representation == "linear":
        X = np.zeros((2 * qni + 1))
        X[::2] = x
        X[1::2] = (x[0:-1] + x[1:]) / 2.0
        Z = np.zeros((qnk + 1, 2 * qni + 1))
        Z[:, 1::2] = z
        Z[:, 2:-1:2] = (z[:, 0:-1] + z[:, 1:]) / 2.0
        Z[:, 0] = z[:, 0]
        Z[:, -1] = z[:, -1]
        Q = np.zeros((qnk, 2 * qni))
        Q[:, ::2] = q
        Q[:, 1::2] = q
    elif representation == "plm":
        X = np.zeros((2 * qni))
        X[::2] = x[:-1]
        X[1::2] = x[1:]
        # PLM reconstruction for Z
        dz = np.roll(z, -1, axis=1) - z  # Right-sided difference
        if not periodicDomain:
            dz[:, -1] = 0  # Non-periodic boundary
        d2 = (
            np.roll(z, -1, axis=1) - np.roll(z, 1, axis=1)
        ) / 2.0  # Centered difference
        d2 = (dz + np.roll(dz, 1, axis=1)) / 2.0  # Centered difference
        s = np.sign(d2)  # Sign of centered slope
        s[dz * np.roll(dz, 1, axis=1) <= 0] = 0  # Flatten extrema
        dz = np.abs(dz)  # Only need magnitude from here on
        S = s * np.minimum(
            np.abs(d2), np.minimum(dz, np.roll(dz, 1, axis=1))
        )  # PLM slope
        Z = np.zeros((qnk + 1, 2 * qni))
        Z[:, ::2] = z - S / 2.0
        Z[:, 1::2] = z + S / 2.0
        Q = np.zeros((qnk, 2 * qni - 1))
        Q[:, ::2] = q
        Q[:, 1::2] = (q[:, :-1] + q[:, 1:]) / 2.0
    else:
        raise Exception("Unknown representation!")

    return X, Z, Q



def ztplot(
    field,
    t=None,
    z=None,
    tlabel=None,
    tunits=None,
    zlabel=None,
    zunits=None,
    splitscale=None,
    title="",
    suptitle="",
    autocenter=False,
    clim=None,
    colormap=None,
    extend=None,
    centerlabels=False,
    nbins=None,
    landcolor=[0.5, 0.5, 0.5],
    aspect=[16, 9],
    resolution=576,
    axis=None,
    ignore=None,
    save=None,
    debug=False,
    show=False,
    interactive=False,
):
    """
    Renders section plot of scalar field, field(x,z).

    Arguments:
    field       Scalar 2D array to be plotted.
    t           t (or time) coordinate (1D array).
    z           z coordinate (1D array).
    tlabel      The label for the t axis. Default 'Time'.
    tunits      The units for the t axis. Default 'Years'.
    zlabel      The label for the z axis. Default 'Elevation'.
    zunits      The units for the z axis. Default 'm'.
    splitscale    A list of depths to define equal regions of projection in the vertical, e.g. [0.,-1000,-6500]
    title       The title to place at the top of the panel. Default ''.
    suptitle    The super-title to place at the top of the figure. Default ''.
    autocenter  If clim generated by script, set to be centered on zero.  Default False.
    clim        A tuple of (min,max) color range OR a list of contour levels. Default None.
    colormap    The name of the colormap to use. Default None.
    extend      Can be one of 'both', 'neither', 'max', 'min'. Default None.
    centerlabels If True, will move the colorbar labels to the middle of the interval. Default False.
    nbins       The number of colors levels (used is clim is missing or only specifies the color range).
    landcolor   An rgb tuple to use for the color of land (no data). Default [.5,.5,.5].
    aspect      The aspect ratio of the figure, given as a tuple (W,H). Default [16,9].
    resolution  The vertical resolution of the figure given in pixels. Default 720.
    axis         The axis handle to plot to. Default None.
    ignore      A value to use as no-data (NaN). Default None.
    save        Name of file to save figure in. Default None.
    debug       If true, report stuff for debugging. Default False.
    show        If true, causes the figure to appear on screen. Used for testing. Default False.
    interactive If true, adds interactive features such as zoom, close and cursor. Default False.
    """

    # Create coordinates if not provided
    tlabel, tunits, zlabel, zunits = createTZlabels(
        t, z, tlabel, tunits, zlabel, zunits
    )
    if debug:
        print("t,z label/units=", tlabel, tunits, zlabel, zunits)
    if ignore is not None:
        maskedField = numpy.ma.masked_array(field, mask=[field == ignore])
    else:
        maskedField = field.copy()
    field2 = maskedField.T
    tCoord = t
    zCoord = z

    # Diagnose statistics
    sMin, sMax, sMean, sStd, sRMS = myStats(maskedField, None, debug=debug)
    tLims = numpy.amin(tCoord), numpy.amax(tCoord)
    zLims = numpy.amin(zCoord), numpy.amax(zCoord)
    # zLims = boundaryStats(zCoord)

    # Choose colormap
    if nbins is None and (clim is None or len(clim) == 2):
        nbins = 35
    if colormap is None:
        colormap = chooseColorMap(sMin, sMax)
    cmap, norm, extend = chooseColorLevels(
        sMin,
        sMax,
        colormap,
        clim=clim,
        nbins=nbins,
        extend=extend,
        autocenter=autocenter,
    )

    if axis is None:
        setFigureSize(aspect, resolution, debug=debug)
        # plt.gcf().subplots_adjust(left=.10, right=.99, wspace=0, bottom=.09, top=.9, hspace=0)
        axis = plt.gca()
    plt.pcolormesh(tCoord, zCoord, field2, cmap=cmap, norm=norm)
    if interactive:
        addStatusBar(tCoord, zCoord, field2)
    cb = plt.colorbar(fraction=0.08, pad=0.02, extend=extend)
    if centerlabels and len(clim) > 2:
        cb.set_ticks(0.5 * (clim[:-1] + clim[1:]))
    axis.set_facecolor(landcolor)
    if splitscale is not None:
        for zzz in splitscale[1:-1]:
            plt.axhline(zzz, color="k", linestyle="--")
        axis.set_yscale("splitscale", zval=splitscale)
    plt.xlim(tLims)
    plt.ylim(zLims)
    axis.annotate(
        "max=%.5g\nmin=%.5g" % (sMax, sMin),
        xy=(0.0, 1.01),
        xycoords="axes fraction",
        verticalalignment="bottom",
        fontsize=10,
    )
    if sMean is not None:
        axis.annotate(
            "mean=%.5g\nrms=%.5g" % (sMean, sRMS),
            xy=(1.0, 1.01),
            xycoords="axes fraction",
            verticalalignment="bottom",
            horizontalalignment="right",
            fontsize=10,
        )
        axis.annotate(
            " sd=%.5g\n" % (sStd),
            xy=(1.0, 1.01),
            xycoords="axes fraction",
            verticalalignment="bottom",
            horizontalalignment="left",
            fontsize=10,
        )
    if len(tlabel + tunits) > 0:
        plt.xlabel(label(tlabel, tunits))
    if len(zlabel + zunits) > 0:
        plt.ylabel(label(zlabel, zunits))
    if len(title) > 0:
        plt.title(title)
    if len(suptitle) > 0:
        plt.suptitle(suptitle)

    if save is not None:
        plt.savefig(save)
    if interactive:
        addInteractiveCallbacks()
    if show:
        plt.show(block=False)



import numpy


def expand(a):
    """
    Expands a vector by one element, averaging the data to the middle columns and
    extrapolating for the first and last rows. Needed for shifting coordinates
    from centers to corners.
    """
    b = numpy.zeros((len(a) + 1))
    b[1:-1] = 0.5 * (a[:-1] + a[1:])
    b[0] = a[0] + 0.5 * (a[0] - a[1])
    b[-1] = a[-1] + 0.5 * (a[-1] - a[-2])
    return b


def expandI(a):
    """
    Expands an array by one column, averaging the data to the middle columns and
    extrapolating for the first and last columns. Needed for shifting coordinates
    from centers to corners.
    """
    nj, ni = a.shape
    b = numpy.zeros((nj, ni + 1))
    b[:, 1:-1] = 0.5 * (a[:, :-1] + a[:, 1:])
    b[:, 0] = a[:, 0] + 0.5 * (a[:, 0] - a[:, 1])
    b[:, -1] = a[:, -1] + 0.5 * (a[:, -1] - a[:, -2])
    return b


def expandJ(a):
    """
    Expands an array by one row, averaging the data to the middle columns and
    extrapolating for the first and last rows. Needed for shifting coordinates
    from centers to corners.
    """
    nj, ni = a.shape
    b = numpy.zeros((nj + 1, ni))
    b[1:-1, :] = 0.5 * (a[:-1, :] + a[1:, :])
    b[0, :] = a[0, :] + 0.5 * (a[0, :] - a[1, :])
    b[-1, :] = a[-1, :] + 0.5 * (a[-1, :] - a[-2, :])
    return b


def boundaryStats(a):
    """
    Returns the minimum and maximum values of a only on the boundaries of the array.
    """
    amin = numpy.amin(a[0, :])
    amin = min(amin, numpy.amin(a[1:, -1]))
    amin = min(amin, numpy.amin(a[-1, :-1]))
    amin = min(amin, numpy.amin(a[1:-1, 0]))
    amax = numpy.amax(a[0, :])
    amax = max(amax, numpy.amax(a[1:, -1]))
    amax = max(amax, numpy.amax(a[-1, :-1]))
    amax = max(amax, numpy.amax(a[1:-1, 0]))
    return amin, amax



import xarray as xr
import numpy as np


grid_1x1 = xr.Dataset()
grid_1x1["lon"] = xr.DataArray(data=np.arange(0.5, 360.5), dims=("lon"))
grid_1x1["lat"] = xr.DataArray(data=np.arange(-89.5, 90.5), dims=("lat"))
grid_1x1["bnds"] = xr.DataArray(data=[0, 1], dims=("bnds"))
grid_1x1["lon_bnds"] = xr.DataArray(
    data=[np.arange(0, 360), np.arange(1, 361)], dims=("bnds", "lon")
)
grid_1x1["lat_bnds"] = xr.DataArray(
    data=[np.arange(-90, 90), np.arange(-89, 91)], dims=("bnds", "lat")
)


def test_infer_and_assign_coord():
    from omlabs.om4common import infer_and_assign_coord

    da = xr.DataArray(data=np.empty((360, 2)), dims=("lon", "bnds"))
    da = infer_and_assign_coord(grid_1x1, da, "lon")
    assert "assigned_lon" in list(da.dims)


def test_copy_coordinates():
    from omlabs.om4common import copy_coordinates

    da = xr.DataArray(data=np.empty((360, 2)), dims=("lon", "bnds"))
    da = copy_coordinates(grid_1x1["lon_bnds"], da, ["lon", "bnds"])
    assert list(da.coords).sort() == list(grid_1x1["lon_bnds"].coords).sort()


def test_compute_area_regular_grid():
    from omlabs.om4common import compute_area_regular_grid

    area = compute_area_regular_grid(grid_1x1)
    assert isinstance(area, np.ndarray)
    assert area.shape == (180, 360)


def test_standardize_longitude():
    from omlabs.om4common import standardize_longitude

    # case 1: extending past 360.
    lon = np.arange(20.5, 380.5, 1.0)
    lat = np.arange(-89.5, 90.5, 1)
    data = np.random.normal(size=(len(lat), len(lon)))
    data = xr.DataArray(
        data, dims={"lat": lat, "lon": lon}, coords={"lat": lat, "lon": lon}
    )
    dset = xr.Dataset({"data": data})

    # 0 to 360
    result = standardize_longitude(dset, "lon")
    assert np.all(dset.sel(lon=370.5).data.values == result.sel(lon=10.5).data.values)
    assert np.all(dset.sel(lon=60.5).data.values == result.sel(lon=60.5).data.values)

    # -180 to 180
    result = standardize_longitude(dset, "lon", start_lon=-180)
    assert np.all(dset.sel(lon=370.5).data.values == result.sel(lon=10.5).data.values)
    assert np.all(dset.sel(lon=209.5).data.values == result.sel(lon=-150.5).data.values)

    # case 2: arbitrary start
    lon = np.arange(-159.5, 200.5, 1.0)
    lat = np.arange(-89.5, 90.5, 1)
    data = np.random.normal(size=(len(lat), len(lon)))
    data = xr.DataArray(
        data, dims={"lat": lat, "lon": lon}, coords={"lat": lat, "lon": lon}
    )
    dset = xr.Dataset({"data": data})

    # 0 to 360
    result = standardize_longitude(dset, "lon")
    assert np.all(dset.sel(lon=-140.5).data.values == result.sel(lon=219.5).data.values)
    assert np.all(dset.sel(lon=60.5).data.values == result.sel(lon=60.5).data.values)

    # -180 to 180
    result = standardize_longitude(dset, "lon", start_lon=-180)
    assert np.all(dset.sel(lon=190.5).data.values == result.sel(lon=-169.5).data.values)
    assert np.all(dset.sel(lon=-60.5).data.values == result.sel(lon=-60.5).data.values)

    # case 3: 2-dimensional coordinates
    x = np.arange(1, 361, 1)
    y = np.arange(1, 181, 1)
    lon = np.arange(-159.5, 200.5, 1.0)
    lat = np.arange(-89.5, 90.5, 1)
    geolon, geolat = np.meshgrid(lon, lat)
    data = np.random.normal(size=geolon.shape)
    geolat = xr.DataArray(geolat, dims={"y": y, "x": x}, coords={"y": y, "x": x})
    geolon = xr.DataArray(geolon, dims={"y": y, "x": x}, coords={"y": y, "x": x})
    data = xr.DataArray(
        data, dims={"y": y, "x": x}, coords={"geolat": geolat, "geolon": geolon}
    )
    dset = xr.Dataset({"data": data})
    result = standardize_longitude(dset, "geolon")
    assert (result.geolon.min() > 0) & (result.geolon.max() < 360)



#!/usr/bin/env python3

import pkg_resources as pkgr
import intake
import io
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from omlabs import m6plot
import palettable
import xarray as xr
import xcompare
import xoverturning
import warnings
from omlabs import m6plot
import gsw as gsw

# these are always needed
from omlabs.om4parser import default_diag_parser
from omlabs.om4common import image_handler

# these are available to you to use
from omlabs.om4common import date_range
from omlabs.om4common import generate_basin_masks
from omlabs.om4common import horizontal_grid
from omlabs.om4common import is_symmetric
from omlabs.om4common import open_intake_catalog
from omlabs.om4common import read_topography
from omlabs.om4common import standardize_longitude


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments
    """

    description = " "

    parser = default_diag_parser(description=description, template=template)

    parser.add_argument(
        "--dataset",
        type=str,
        required=False,
        default="WOA13_annual_TS",
        help="Name of the observational dataset, \
              as provided in intake catalog",
    )

    parser.add_argument(
        "--argo_file",
        type=str,
        required=False,
        default=None,
        help="Name of the Argo climatology file "
        + "(in-situ temp and practical salinity)",
    )

    parser.add_argument(
        "--model_xcoord",
        type=str,
        required=False,
        default="lon",
        help="Name of x-coordinate in the model data",
    )

    parser.add_argument(
        "--model_ycoord",
        type=str,
        required=False,
        default="lat",
        help="Name of y-coordinate in the model data",
    )

    parser.add_argument(
        "--model_zcoord",
        type=str,
        required=False,
        default="z_l",
        help="Name of z-coordinate in the model data",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(
    dictArgs,
    tempvar="thetao",
    saltvar="so",
    argo_xcoord="lon",
    argo_ycoord="lat",
    argo_tvar="ptemp",
    argo_svar="salt",
    argo_pvar="pres",
):
    """Read required fields

    Parameters
    ----------
    dictArgs : dict
        Dictionary containing argparse options
    tempvar : str, optional
        Name of potentital temperature variable, by default "thetao"
    saltvar : str, optional
        Name of practical salinity variable, by default "so"
    argo_xcoord : str, optional
        Name of longitude coordinate in obs dataset, by default "lon"
    argo_ycoord : str, optional
        Name of latitude coordinate in obs dataset, by default "lat"
    argo_tvar : str, optional
        Name of temperature variable in obs dataset, by default "ptemp"
    argo_svar : str, optional
        Name of salinity variable in obs dataset, by default "salt"
    argo_pvar : str, optional
        Name of pressure variable in obs dataset, by default "pres"

    Returns
    -------
    xarray.DataSet
    """

    model_xcoord = dictArgs["model_xcoord"]
    model_ycoord = dictArgs["model_ycoord"]
    model_zcoord = dictArgs["model_zcoord"]

    model = xr.open_mfdataset(dictArgs["infile"], use_cftime=True)
    model = xr.Dataset(
        {
            "temp": model[tempvar].mean(dim="time"),
            "salt": model[saltvar].mean(dim="time"),
        }
    )

    # clean up and standardize
    model = model.squeeze()
    model = model.reset_coords(drop=True)
    model = model.rename({model_xcoord: "lon", model_ycoord: "lat"})

    # ----- read the Argo data

    if dictArgs["argo_file"] is not None:
        argo_dset = xr.open_dataset(dictArgs["argo_file"], decode_times=False,)
    else:
        # use dataset from catalog, either from command line or default
        cat = open_intake_catalog(dictArgs["platform"], "obs")
        argo_dset = cat["Argo_Climatology"].to_dask()

    # standardize longitude to go from 0 to 360.
    argo_dset = standardize_longitude(argo_dset, argo_xcoord)

    # subset varaibles
    argo_dset = xr.Dataset(
        {
            "temp": argo_dset[argo_tvar],
            "salt": argo_dset[argo_svar],
            "pres": argo_dset[argo_pvar],
        }
    )

    # clean up and standardize
    argo_dset = argo_dset.squeeze()
    argo_dset = argo_dset.reset_coords(drop=True)
    argo_dset = argo_dset.rename({argo_xcoord: "lon", argo_ycoord: "lat"})

    # ----- read the WOA data

    if dictArgs["obsfile"] is not None:
        # priority to user-provided obs file
        dsobs = xr.open_mfdataset(
            dictArgs["obsfile"], combine="by_coords", decode_times=False
        )
    else:
        # use dataset from catalog, either from command line or default
        cat = open_intake_catalog(dictArgs["platform"], "obs")
        dsobs = cat[dictArgs["dataset"]].to_dask()

    woa = xr.Dataset({"temp": dsobs["ptemp"], "salt": dsobs["salinity"],})
    woa = woa.squeeze()
    woa = woa.reset_coords(drop=True)

    return model, woa, argo_dset


def calculate(
    model,
    woa,
    argo_dset,
    zcoord="z_l",
    model_xcoord="lon",
    model_ycoord="lat",
    argo_xcoord="lon",
    argo_ycoord="lat",
    argo_zcoord="pres",
    depth_range=(25, 1000),
):
    """Main calculation function

    Parameters
    ----------
    dset : xarray.Dataset
        Input dataset
    dset_static : xarray.Dataset
        Input static dataset

    Returns
    -------
    xarray.DataArray
    """

    # -- Model Data

    pottemp = model["temp"]

    dtdz = (-1.0 * pottemp).differentiate(zcoord).sel({zcoord: slice(*depth_range)})
    dtdz.load()
    mask = xr.where(dtdz.isel({zcoord: 0}).isnull(), np.nan, 1.0)
    dtdz = dtdz.fillna(0.0)
    dtdz_max = dtdz.max(zcoord)
    dtdz_max_depth = xr.broadcast(dtdz.z_l, dtdz)[0][dtdz.argmax(zcoord)]
    dtdz_model_max = xr.Dataset(
        {"dtdz_max": dtdz_max * mask, "dtdz_max_depth": dtdz_max_depth * mask}
    )

    # define regions for model data
    # _xval = pottemp[model_xcoord].sel({model_xcoord: -140.0}, method="nearest")
    _xval = pottemp[model_xcoord].sel({model_xcoord: 220.0}, method="nearest")
    pacific = {model_xcoord: _xval, model_ycoord: slice(-70.0, 60.0)}
    # _xval = pottemp[model_xcoord].sel({model_xcoord: -30.0}, method="nearest")
    _xval = pottemp[model_xcoord].sel({model_xcoord: 330.0}, method="nearest")
    atlantic = {model_xcoord: _xval, model_ycoord: slice(-70.0, 80.0)}

    # extract sections for comparison with Argo
    basin = xr.DataArray(np.array(["atlantic", "pacific"]), dims=("basin"))
    model = xr.concat([pottemp.sel(atlantic), pottemp.sel(pacific)], dim="basin")
    model = model.assign_coords({"basin": basin})

    dtdz_model = (-1.0 * model).differentiate(zcoord).squeeze()

    # -- WOA

    dtdz = (-1.0 * woa.temp).differentiate(zcoord).sel({zcoord: slice(*depth_range)})
    dtdz.load()
    mask = xr.where(dtdz.isel({zcoord: 0}).isnull(), np.nan, 1.0)
    dtdz = dtdz.fillna(0.0)
    dtdz_max = dtdz.max(zcoord)
    dtdz_max_depth = xr.broadcast(dtdz.z_l, dtdz)[0][dtdz.argmax(zcoord)]
    dtdz_woa_max = xr.Dataset(
        {"dtdz_max": dtdz_max * mask, "dtdz_max_depth": dtdz_max_depth * mask}
    )

    # -- ARGO

    _xval = argo_dset[argo_xcoord].sel({argo_xcoord: 220.0}, method="nearest")
    pacific = {argo_xcoord: _xval, argo_ycoord: slice(-70.0, 60.0)}

    _xval = argo_dset[argo_xcoord].sel({argo_xcoord: 330.0}, method="nearest")
    atlantic = {argo_xcoord: _xval, argo_ycoord: slice(-70.0, 80.0)}

    argo_dset = xr.concat(
        [argo_dset.sel(atlantic), argo_dset.sel(pacific)], dim="basin"
    )
    argo_dset = argo_dset.assign_coords({"basin": basin})

    argo_dset = xr.Dataset(
        {
            "thetao": xr.DataArray(
                gsw.pt_from_t(
                    argo_dset["salt"], argo_dset["temp"], argo_dset["pres"], 0.0
                ),
                name="thetao",
                attrs={"long_name": "Seawater Potential Temperature", "units": "deg C"},
            )
        }
    )

    dtdz_argo = (-1.0 * argo_dset.thetao).differentiate(argo_zcoord).squeeze()

    max_comparison_results = xcompare.compare_datasets(dtdz_model_max, dtdz_woa_max)

    return max_comparison_results, dtdz_model, dtdz_argo


def _plot_basin(
    dtdz_model, dtdz_argo, basin="atlantic", model_zcoord="z_l", argo_zcoord="pres"
):

    cmap = plt.cm.RdYlBu_r
    levels = [
        1.0e-7,
        5.0e-7,
        1.0e-6,
        5.0e-6,
        1.0e-5,
        5.0e-5,
        1.0e-4,
        5.0e-4,
        1.0e-3,
        5.0e-3,
    ] + list(np.linspace(0.01, 0.2, 10))
    norm = mpl.colors.BoundaryNorm(levels, ncolors=cmap.N, clip=False)

    fig = plt.figure(figsize=(18, 5))
    ax1 = plt.subplot(1, 2, 1, facecolor="gray")
    plotarr = dtdz_model.sel(basin=basin)
    cb = ax1.pcolormesh(
        plotarr.lat,
        plotarr[model_zcoord],
        plotarr.values,
        cmap=cmap,
        norm=norm,
        shading="auto",
    )
    plt.plot(plotarr.lat, (plotarr.lat * 0.0) + 2000, "k--")
    plt.colorbar(cb)
    ax1.set_yscale("splitscale", zval=[5500, 2000, 0])
    ax1.set_xlim(-70, 70)

    ax2 = plt.subplot(1, 2, 2, facecolor="gray")
    plotarr = dtdz_argo.sel(basin=basin)
    cb = ax2.pcolormesh(
        plotarr.lat,
        plotarr[argo_zcoord],
        plotarr.values,
        cmap=cmap,
        norm=norm,
        shading="auto",
    )
    plt.plot(plotarr.lat, (plotarr.lat * 0.0) + 2000, "k--")
    plt.colorbar(cb)
    ax2.set_yscale("splitscale", zval=[5500, 2000, 0])
    ax2.set_xlim(-70, 70)

    ax1.text(
        0.01,
        1.02,
        f"a. {basin.capitalize()} Sector dT/dz",
        ha="left",
        transform=ax1.transAxes,
    )
    ax1.text(0.98, 1.02, f"Model", ha="right", transform=ax1.transAxes)
    ax2.text(
        0.01,
        1.02,
        f"a. {basin.capitalize()} Sector dT/dz",
        ha="left",
        transform=ax2.transAxes,
    )
    ax2.text(0.98, 1.02, f"Argo", ha="right", transform=ax2.transAxes)

    return fig


def plot(max_comparison_results, dtdz_model, dtdz_argo):
    """Plotting wrapper

    Parameters
    ----------
    arr : xarray.DataArray
        Input data array

    Returns
    -------
    matplotlib.figure.Figure
        Figure handle
    """

    figs = [
        xcompare.plot_three_panel(
            max_comparison_results, "dtdz_max", diffvmin=-0.02, diffvmax=0.02
        ),
        xcompare.plot_three_panel(
            max_comparison_results, "dtdz_max_depth", diffvmin=-150, diffvmax=150
        ),
    ]

    figs.append(_plot_basin(dtdz_model, dtdz_argo, "atlantic"))
    figs.append(_plot_basin(dtdz_model, dtdz_argo, "pacific"))

    return figs


def run(dictArgs):
    """Function to call read, calc, and plot in sequence

    Parameters
    ----------
    dictArgs : dict
        Dictionary of parsed options

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # read in data
    model, woa, argo_dset = read(dictArgs)

    # calculate
    max_comparison_results, dtdz_model, dtdz_argo = calculate(model, woa, argo_dset)

    # make the plots
    figs = plot(max_comparison_results, dtdz_model, dtdz_argo)

    filename = [
        f"{dictArgs['outdir']}/thermocline_strength",
        f"{dictArgs['outdir']}/thermocline_depth",
        f"{dictArgs['outdir']}/argo_comparison_atlantic",
        f"{dictArgs['outdir']}/argo_comparison_pacific",
    ]
    imgbufs = image_handler(figs, dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Parameters
    ----------
    cliargs : argparse, optional
        command line arguments from upstream instance, by default None

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



import xarray as xr

""" A set of helper functions """


def try_variable_from_list(vars_in_file, query_vars):
    """test variables from query_vars to find which one
    is present in vars_in_file"""
    for var in query_vars:
        if var in vars_in_file:
            return var
    return None


def get_run_name(ncfile):
    ds = xr.open_mfdataset(ncfile, combine="by_coords")
    if "title" in ds.attrs:
        return ds.attrs["title"]
    else:
        return "Unknown experiment"



import numpy
import numpy.matlib


def yzWeight(y, z):
    """
    Calculates the weights to use when calculating the statistics of a y-z section.

    y(nj+1) is a 1D vector of column edge positions and z(nk+1,nj) is the interface
    elevations of each column. Returns weight(nk,nj).
    """
    dz = z[:-1, :] - z[1:, :]
    return numpy.matlib.repmat(y[1:] - y[:-1], dz.shape[0], 1) * dz



import numpy


def expandJ(a):
    """
    Expands an array by one row, averaging the data to the middle columns and
    extrapolating for the first and last rows. Needed for shifting coordinates
    from centers to corners.
    """
    nj, ni = a.shape
    b = numpy.zeros((nj + 1, ni))
    b[1:-1, :] = 0.5 * (a[:-1, :] + a[1:, :])
    b[0, :] = a[0, :] + 0.5 * (a[0, :] - a[1, :])
    b[-1, :] = a[-1, :] + 0.5 * (a[-1, :] - a[-2, :])
    return b



""" omlabs lexicon for default diagnostic CLI options """

import argparse


class DefaultDictParser(argparse.ArgumentParser):
    """Custom argparse class extension capable of returning a dictionary
    object. This class will bypass the error when called without
    required options and will prepoulate with default values

    Parameters
    ----------
    argparse : argparse.ArgumentParser

    See Also
    --------
    default_diag_parser : function to return default options
    """

    def error(self, message):
        actions = self.__dict__["_actions"]
        defaults = {}
        for act in actions[1::]:
            defaults[act.__dict__["dest"]] = act.__dict__["default"]
        return defaults


def default_diag_parser(description="", template=False, exclude=None):
    """Establishes a default ArgumentParser object with default values.
    Options may be omitted if desired. The default set includes:

    Baseline options:

    These options control common framework functions:

    INFILE:  path; dataset, list of datasets, or regular expression
    pointing to input files

    -l, --label:        str; string to use in plot titles
                        (typically the experiment name)
    -o, --outdir:       path; directory to store plots.  Will create if not present
    -d, --dpi:          int; figure resolution in dots per inch (default is 100)
    -i, --interactive:  bool; displays figures to screen if present
    -v, --verbose:      turn on verbose output.  Need to consider multiple
                        levels of verbosity
    -S, --suptitle:     str; super-title to annotate figures

    Grid options:

    The horizontal and vertical grid information is contained in the gridSpec tar file
    and will be the preferred route for obtaining this data.  Some grids, such as the
    1x1 spherical grid, do not have a gridSpec tar file.  In this case, an
    ocean.static dataset may be provided

    -g, --gridspec:     path;  gridspec tarfile
    -s, --static:       path;  ocean.static dataset

    Intake catalogs:

    Intake catalogs will be used to specify default grid files for known model
    configurations and for paths to observational datasets.  These will be the
    preferred paths:

    -c, --config:       str; model configuration (e.g. OM4p25, OM4p5, OM4p125)
    -p, --platform:     str; name of system. Used to find intake catalogs
                        (e.g. gfdl, orion, gaea)

    Overrides:

    Options are available to override the intake catalog values.  Overrides will
    be defined in long format, e.g. no short single letter options:

    --obsfile:      path; observational file
    --topog:        path; topography file
    --hgrid:        path; ocean hgrid file
    --basin:        path; file containing basin codes


    Parameters
    ----------
    description : str, optional
        descriptive string to pass to the CLI, by default ""
    template : bool, optional
        return a DefaultDictParser object if True, by default False
    exclude : str, list, optional
        str or list of str of options to exclude, by default None

    Returns
    -------
    ArgumentParser
        pre-populated ArgumentParser object with default opts

    See Also
    --------
    DefaultDictParser : custom class capable of returning a dict() of CLI args

    """
    if exclude is not None:
        if not isinstance(exclude, list):
            exclude = [exclude]
    else:
        exclude = []

    if template is True:
        parser = DefaultDictParser(
            description=description, formatter_class=argparse.RawTextHelpFormatter
        )
    else:
        parser = argparse.ArgumentParser(
            description=description, formatter_class=argparse.RawTextHelpFormatter
        )

    # baseline options
    if "infile" not in exclude:
        parser.add_argument(
            "infile",
            metavar="INFILE",
            type=str,
            nargs="+",
            help="Path to input NetCDF file(s)",
        )

    if "label" not in exclude:
        parser.add_argument(
            "-l",
            "--label",
            type=str,
            default="",
            help="String label to annotate the plot",
        )

    if "outdir" not in exclude:
        parser.add_argument(
            "-o",
            "--outdir",
            type=str,
            default="./",
            help="Output directory. Default is current directory",
        )

    if "dpi" not in exclude:
        parser.add_argument(
            "-d",
            "--dpi",
            type=int,
            default=100,
            help="Figure resolution in dots per inch. Default is 100.",
        )

    if "interactive" not in exclude:
        parser.add_argument(
            "-i",
            "--interactive",
            action="store_true",
            help="Interactive mode displays plot to screen. Default is False",
        )

    if "verbose" not in exclude:
        parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            help="Verbose output",
        )

    if "suptitle" not in exclude:
        parser.add_argument(
            "-S",
            "--suptitle",
            type=str,
            default="",
            help="Super-title for annotating figures",
        )

    if "format" not in exclude:
        parser.add_argument(
            "-f",
            "--format",
            type=str,
            default="png",
            help="Output format for plots. Default is png",
        )

    # grid-related options
    if "gridspec" not in exclude:
        parser.add_argument(
            "-g", "--gridspec", type=str, default=None, help="Path to gridspec tar file"
        )

    if "static" not in exclude:
        parser.add_argument(
            "-s",
            "--static",
            type=str,
            default=None,
            help="Path to ocean.static.nc file",
        )

    # intake catalog-related options
    if "config" not in exclude:
        parser.add_argument(
            "-c",
            "--config",
            type=str,
            default=None,
            help="Model configuration, default is OM4",
        )

    if "platform" not in exclude:
        parser.add_argument(
            "--platform",
            type=str,
            required=False,
            default="gfdl",
            help="Computing platform, default is gfdl",
        )

    if "basin" not in exclude:
        # override arguments
        parser.add_argument(
            "--basin",
            type=str,
            default=None,
            help="Path to basin code file",
        )

    if "obsfile" not in exclude:
        parser.add_argument(
            "--obsfile",
            type=str,
            default=None,
            help="Path to observational dataset file",
        )

    if "hgrid" not in exclude:
        parser.add_argument(
            "--hgrid",
            type=str,
            default=None,
            help="Path to hgrid file",
        )

    if "topog" not in exclude:
        parser.add_argument(
            "--topog",
            type=str,
            default=None,
            help="Path to topography file",
        )

    return parser



def createXYlabels(x, y, xlabel, xunits, ylabel, yunits):
    """
    Checks that x and y labels are appropriate and tries to make some if they are not.
    """
    if x is None:
        if xlabel is None:
            xlabel = "i"
        if xunits is None:
            xunits = ""
    else:
        if xlabel is None:
            xlabel = "Longitude"
        # if xunits is None: xunits=u'\u00B0E'
        if xunits is None:
            xunits = r"$\degree$E"
    if y is None:
        if ylabel is None:
            ylabel = "j"
        if yunits is None:
            yunits = ""
    else:
        if ylabel is None:
            ylabel = "Latitude"
        # if yunits is None: yunits=u'\u00B0N'
        if yunits is None:
            yunits = r"$\degree$N"
    return xlabel, xunits, ylabel, yunits



def drawNinoBoxes(m, region="all"):
    """
    Function to draw ENSO region boxes on a basemap instance
    """
    if region == "nino4" or region == "all":
        polyLon = [-200.0, -200.0, -150.0, -150.0, -200.0]
        polyLat = [-5.0, 5.0, 5.0, -5.0, -5.0]
        polyX, polyY = m(polyLon, polyLat)
        m.plot(polyX, polyY, marker=None, color="k", linewidth=2.0)
    if region == "nino3" or region == "all":
        polyLon = [-150.0, -150.0, -90.0, -90.0, -150.0]
        polyLat = [-5.0, 5.0, 5.0, -5.0, -5.0]
        polyX, polyY = m(polyLon, polyLat)
        m.plot(polyX, polyY, marker=None, color="k", linewidth=2.0)
    if region == "nino34" or region == "all":
        polyLon = [-170.0, -170.0, -120.0, -120.0, -170.0]
        polyLat = [-5.0, 5.0, 5.0, -5.0, -5.0]
        polyX, polyY = m(polyLon, polyLat)
        m.plot(polyX, polyY, marker=None, color="r", linestyle="dashed", linewidth=2.0)
    if region == "nino12" or region == "all":
        polyLon = [-90.0, -90.0, -80.0, -80.0, -90.0]
        polyLat = [-10.0, 0.0, 0.0, -10.0, -10.0]
        polyX, polyY = m(polyLon, polyLat)
        m.plot(polyX, polyY, marker=None, color="k", linewidth=2.0)



def label(label, units):
    """
    Combines a label string and units string together in the form 'label [units]'
    unless one of the other is empty.
    """
    string = r"" + label
    if len(units) > 0:
        string = string + " [" + units + "]"
    return string



def createYZlabels(y, z, ylabel, yunits, zlabel, zunits):
    """
    Checks that y and z labels are appropriate and tries to make some if they are not.
    """
    if y is None:
        if ylabel is None:
            ylabel = "j"
        if yunits is None:
            yunits = ""
    else:
        if ylabel is None:
            ylabel = "Latitude"
        # if yunits is None: yunits=u'\u00B0N'
        if yunits is None:
            yunits = r"$\degree$N"
    if z is None:
        if zlabel is None:
            zlabel = "k"
        if zunits is None:
            zunits = ""
    else:
        if zlabel is None:
            zlabel = "Elevation"
        if zunits is None:
            zunits = "m"
    return ylabel, yunits, zlabel, zunits



def test_try_variable_from_list():
    from omlabs.helpers import try_variable_from_list

    vars_in_file = ["thetao", "so"]
    query_vars = ["temp", "ptemp", "thetao", "TEMP"]

    out = try_variable_from_list(vars_in_file, query_vars)

    assert isinstance(out, str)
    assert out == "thetao"

    query_vars = ["temp", "ptemp", "TEMP"]

    out = try_variable_from_list(vars_in_file, query_vars)

    assert out is None



from .moc import parse, read, calculate, plot, run, parse_and_run

__description__ = "Plots global and basin-average overturning streamfunction"
__ppstreams__ = [
    "ocean_month_z_d2_refined/av",
    "ocean_monthly_z_d2/ts",
    "ocean_annual_z/av",
]
__ppvars__ = ["vmo"]



import xarray as xr
import matplotlib.pyplot as plt

from omlabs.om4common import image_handler
from omlabs.om4common import open_intake_catalog
from omlabs.om4parser import default_diag_parser

# Define a list of acceptable coordinates
possible_xdims = ["xq", "xh", "xc", "lon", "xq_sub01", "xq_sub02"]
possible_ydims = ["yh", "yq", "yc", "lat", "yh_sub01", "yh_sub02"]
possible_zdims = ["z_l", "lev", "level"]


def parse(cliargs=None, template=False):
    description = """Plot annual volume transport through the Drake Passage"""

    parser = default_diag_parser(description=description, template=template)

    parser.add_argument("--dataset", type=str, required=False)

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs):
    """read zonal velocity field from list of files (either global or section) and extract drake passage if needed"""

    # open the dataset
    dset = xr.open_mfdataset(dictArgs["infile"])

    # Read in umo from dataset to data array
    darray = dset["umo"]

    # set converts our list to a squence of distinct interable elements
    # the intersection method returns a set that contains the items
    # which exist in both set(darray.dims) and set(possible_xdims)
    # and then we turn this back into a list
    xdim = list(set(darray.dims).intersection(set(possible_xdims)))
    ydim = list(set(darray.dims).intersection(set(possible_ydims)))
    zdim = list(set(darray.dims).intersection(set(possible_zdims)))

    # Make sure we have exactly one value per coordinate
    # Immediately trigger error message if condition is false.
    for dim in [xdim, ydim, zdim]:
        assert len(dim) == 1, "Ambiguous coordinates found."

    xdim = xdim[0]
    ydim = ydim[0]
    zdim = zdim[0]

    # If the max latitude is positive (i.e. Northern Hemisphere), assume
    # we have a global array that needs to be subset. Otherwise, just keep
    # the array as-is
    # RD: this will require fix when switching from nominal coords to grid indices

    if max(darray[ydim]) > 0.0:
        darray = darray.sel({xdim: -70, ydim: slice(-70, -54)}, method="nearest")
    else:
        darray

    return darray


def calculate(darray):
    """Performs the ACC calculation by slicing to correct
       y and x regions if using global umo output. Calls
       the ACC_Transport function to compute the transport.

    Parameters
    ----------
    darray : xarray.DataArray
        Data Array containing transport

    Returns
    -------
    xarray.DataArray
        Output DataArray with time series of ACC transport
    """

    dims = list(darray.dims)
    dims.remove("time")

    full_transport = darray.sum(dim=dims)
    full_transport = full_transport * (1.0 / 1035.0) * 1.0e-6  # convert to Sv
    annual_mean_transport = full_transport.groupby("time.year").mean(dim="time")

    return annual_mean_transport


def plot(darray_ts):
    """plot the timeserie of ACC transport at Drake passage vs observations

    Parameters
    ----------
    darray : xarray.DataArray
        Data Array containing time series of ACC transport

    Returns
    -------
    figure object
    """

    ## Donohue et al. 2016 mean +/- 2 sigma
    x_end = len(darray_ts.values)
    x_full = (0, len(darray_ts.values))
    max_Donohue = 194.4
    min_Donohue = 151.9
    mean_Donohue = 173.3

    ## Xu et al. 2020 mean +/- 2 sigma
    max_Xu = 161.9
    min_Xu = 152.7
    mean_Xu = 157.3

    ## Biogeochemical Southern Ocean State Estimate (BSOSE) 1/6 degree
    mean_BSOSE = 164

    fig = plt.figure(figsize=(10, 5))
    plt.plot(darray_ts.values)
    plt.ylabel("ACC [Sv]", fontweight="bold", fontsize="large")
    plt.xlabel("Model Year", fontweight="bold", fontsize="large")
    plt.grid()

    plt.fill_between(x_full, min_Donohue, max_Donohue, facecolor="grey", alpha=0.5)
    plt.fill_between(x_full, min_Xu, max_Xu, facecolor="yellow", alpha=0.5)
    plt.axhline(y=mean_Donohue, linestyle="--", color="grey")
    plt.axhline(y=mean_Xu, linestyle="--", color="yellow")
    plt.plot(
        x_end,
        mean_Donohue,
        marker="o",
        markersize=10,
        markerfacecolor="grey",
        markeredgecolor="k",
    )
    plt.plot(
        x_end,
        mean_Xu,
        marker="o",
        markersize=10,
        markerfacecolor="yellow",
        markeredgecolor="k",
    )
    plt.plot(
        x_end,
        mean_BSOSE,
        marker="o",
        markersize=10,
        markerfacecolor="cyan",
        markeredgecolor="k",
    )

    fig.text(
        0.92,
        0.67,
        "Donohue et al. 2016 (cDrake)",
        fontsize=12,
        color="black",
        fontweight="bold",
    )
    fig.text(0.92, 0.60, "BSOSE 1/6th", fontsize=12, color="black", fontweight="bold")
    fig.text(
        0.92,
        0.55,
        "Xu et al. 2021 (1/12th HYCOM)",
        fontsize=12,
        color="black",
        fontweight="bold",
    )

    return fig


def run(dictArgs):
    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    darray = read(dictArgs)
    darray_ts = calculate(darray)
    figs = plot(darray_ts)
    figs = [figs] if not isinstance(figs, list) else figs
    assert isinstance(figs, list), "Figures must be inside a list object"

    filenames = [
        f"{dictArgs['outdir']}/Volume_Transport_Drake",
    ]

    imgbufs = image_handler(figs, dictArgs, filename=filenames)

    return imgbufs


def parse_and_run(cliargs=None):
    """Function to make compatibile with the superwrapper"""
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



#!/usr/bin/env python3

import argparse
import glob
import io
import os
import matplotlib as mpl
import matplotlib.pyplot as plt
from omlabs import m6plot
import xarray as xr

from omlabs.diags import generic_section_transport
from omlabs.om4common import image_handler
from omlabs.om4common import discover_ts_dir
from omlabs.om4parser import default_diag_parser

# passages with their frepp component mappings and observed ranges from
# Griffies et al., 2016: OMIP contribution to CMIP6:
#    experimental and diagnostic protocol for the physical component
#    of the Ocean Model Intercomparison Project.
#    Geosci. Model. Dev., 9, 3231-3296. doi:10.5194/gmd-9-3231-2016'

reference = (
    "Griffies et al., 2016, "
    + "Geosci. Model. Dev., 9, 3231-3296. "
    + "doi:10.5194/gmd-9-3231-2016"
)

# passages below are (pp driectory, text label, depth range, obs limit)
defined_passages = [
    ("ocean_Agulhas_section", "Agulhas", None, (129.8, 143.6)),
    ("ocean_Bering_Strait", "Bering Strait", None, (0.7, 1.1)),
    ("ocean_Barents_opening", "Barents Opening", None, (1.99, 2.01)),
    ("ocean_Davis_Strait", "Davis Strait", None, (-2.1, -1.1)),
    ("ocean_Denmark_Strait", "Denmark Strait", None, (-4.8, -2.0)),
    ("ocean_Drake_Passage", "Drake Passage", None, (129.8, 143.6)),
    ("ocean_English_Channel", "English Channel", None, (0.01, 0.1)),
    ("ocean_Faroe_Scotland", "Faroe-Scotland", None, (0.8, 1.0)),
    ("ocean_Florida_Bahamas", "Florida-Bahamas", None, (28.9, 34.3)),
    ("ocean_Fram_Strait", "Fram Strait", None, (-4.7, 0.7)),
    ("ocean_Gibraltar_Strait", "Gibraltar Strait", None, (0.109, 0.111)),
    (
        ["ocean_Iceland_Faroe_U", "ocean_Iceland_Faroe_V"],
        "Iceland-Faroe",
        None,
        (4.35, 4.85),
    ),
    ("ocean_Iceland_Norway", "Iceland-Norway", None, None),
    ("ocean_Indonesian_Throughflow", "Indonesian Throughflow", None, (-15, -13)),
    ("ocean_Mozambique_Channel", "Mozambique Channel", None, (-25.6, -7.8)),
    (
        "ocean_Pacific_undercurrent",
        "Pacific Equatorial Undercurrent",
        (0.0, 350.0),
        (24.5, 28.3),
    ),
    ("ocean_Taiwan_Luzon", "Taiwan-Luzon Strait", None, (-3.0, -1.8)),
    ("ocean_Windward_Passage", "Windward Passage", None, (-15, 5)),
]


class Passage:
    """Class to hold passage metadata and associate a plotting function"""

    def __init__(
        self,
        label,
        pathpp,
        tscomponent,
        passage_label="",
        zlim=None,
        obsrange=None,
        varlist=["umo", "vmo"],
    ):

        # mutliple pp components may be passed if there are separate
        # u and v components for a transport
        tscomponent = (
            [tscomponent] if not isinstance(tscomponent, list) else tscomponent
        )
        tscomponent = [f"{pathpp}/{x}" for x in tscomponent]
        tscomponent = [x for x in tscomponent if os.path.exists(x)]
        pathpp = [discover_ts_dir(x) for x in tscomponent]

        # resolve paths to the files to use for plotting
        files = [glob.glob(f"{directory}/*.nc") for directory in pathpp]
        files = [filename for sublist in files for filename in sublist]
        files = [
            filename
            for filename in files
            if any(variable in filename for variable in varlist)
        ]

        self.files = files
        self.passage_label = passage_label
        self.label = label
        self.zlim = zlim
        self.obsrange = obsrange

    def calculate(self):
        if len(self.files) == 0:
            # use print instead of exception to print message to Dora
            print(f"No files found to calculate {self.passage_label}")
            self.transport = None

        else:
            # invoke generic section transport diagnostic
            dset_transport = xr.open_mfdataset(self.files, use_cftime=True)
            self.transport = generic_section_transport.calculate(
                dset_transport, zlim=self.zlim
            )

        return self

    def plot(self):
        if self.transport is not None:
            fig = generic_section_transport.plot(
                self.transport,
                label=self.label,
                passage_label=self.passage_label,
                obsrange=self.obsrange,
            )
            return fig


def parse(cliargs=None, template=False):
    """
    Function to capture the user-specified command line options
    """
    description = """Driver script for plotting all OM4 section transports"""

    exclude = [
        "basin",
        "config",
        "gridspec",
        "hgrid",
        "infile",
        "obsfile",
        "platform",
        "static",
        "suptitle",
        "topog",
    ]

    parser = default_diag_parser(
        description=description, template=template, exclude=exclude
    )

    parser.add_argument(
        "--netcdf",
        type=str,
        default=None,
        help="Name of NetCDF file containing passage timeseries. Default is None.",
    )

    parser.add_argument(
        "ppdir",
        metavar="PPDIR",
        type=str,
        nargs="+",
        help="Path to root post-processing directory (<...>/pp)",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def run(dictArgs):
    """Function to call read, calc, and plot in sequence"""

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")
    else:
        plt.switch_backend("TkAgg")

    # --- the main show ---

    assert len(dictArgs["ppdir"]) == 1, "Only one path may be provided to this routine"
    ppdir = dictArgs["ppdir"][0]

    # read in the data
    passages = [Passage(dictArgs["label"], ppdir, *x) for x in defined_passages]

    # calculate the transports for each of the passages
    passages = [x.calculate() for x in passages]

    # create a unifying xarray.Dataset object
    ds_out = xr.Dataset()
    for x in passages:
        if x.transport is not None:
            outvarname = x.passage_label.replace(" ", "_").replace("-", "_")
            ds_out[outvarname] = x.transport

    # save timeseries to netcdf format if requested
    outfile = ds_out.to_netcdf()
    if dictArgs["netcdf"] is not None:
        with open(f"{dictArgs['outdir']}/{dictArgs['netcdf']}", "wb") as fhandle:
            fhandle.write(outfile)

    # loop over passages and call their plot method
    figs = [passage.plot() for passage in passages if passage.transport is not None]

    # get a list of output filenames
    filenames = [
        passage.passage_label for passage in passages if passage.transport is not None
    ]

    # ---------------------

    filenames = [x.replace(" ", "_") for x in filenames]
    filenames = [f"{dictArgs['outdir']}/{x}" for x in filenames]
    imgbufs = image_handler(figs, dictArgs, filename=filenames)

    return (imgbufs, outfile)


def parse_and_run(cliargs=None):
    args = parse(cliargs)
    args = args.__dict__
    imgbuf, outfile = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()
